@@ -38,7 +38,7 @@
 
 #include "kdc_locl.h"
 
-RCSID("$Id: connect.c,v 1.54 1999/04/01 18:25:54 joda Exp $");
+RCSID("$Id: connect.c,v 1.67 1999/10/06 12:54:23 assar Exp $");
 
 struct port_desc{
     int family;
@@ -50,22 +50,11 @@ static struct port_desc *ports;
 static int num_ports;
 
 static void
-add_port(int family, const char *port_str, const char *protocol)
+add_port(int family, int port, const char *protocol)
 {
-    struct servent *sp;
     int type;
-    int port;
     int i;
 
-    sp = roken_getservbyname(port_str, protocol);
-    if(sp){
-	port = sp->s_port;
-    }else{
-	char *end;
-	port = htons(strtol(port_str, &end, 0));
-	if(end == port_str)
-	    return;
-    }
     if(strcmp(protocol, "udp") == 0)
 	type = SOCK_DGRAM;
     else if(strcmp(protocol, "tcp") == 0)
@@ -85,19 +74,47 @@ add_port(int family, const char *port_str, const char *protocol)
     num_ports++;
 }
 
+static void
+add_port_service(int family, const char *service, int port,
+		 const char *protocol)
+{
+    port = krb5_getportbyname (context, service, protocol, port);
+    add_port (family, port, protocol);
+}
+
+static void
+add_port_string (int family, const char *port_str, const char *protocol)
+{
+    struct servent *sp;
+    int port;
+
+    sp = roken_getservbyname (port_str, protocol);
+    if (sp != NULL) {
+	port = sp->s_port;
+    } else {
+	char *end;
+
+	port = htons(strtol(port_str, &end, 0));
+	if (end == port_str)
+	    return;
+    }
+    add_port (family, port, protocol);
+}
+
 static void
 add_standard_ports (int family)
 {
-    add_port(family, "kerberos", "udp");
-    add_port(family, "kerberos", "tcp");
-    add_port(family, "kerberos-sec", "udp");
-    add_port(family, "kerberos-sec", "tcp");
-    add_port(family, "kerberos-iv", "udp");
-    add_port(family, "kerberos-iv", "tcp");
+    add_port_service(family, "kerberos", 88, "udp");
+    add_port_service(family, "kerberos", 88, "tcp");
+    add_port_service(family, "kerberos-sec", 88, "udp");
+    add_port_service(family, "kerberos-sec", 88, "tcp");
+    add_port_service(family, "kerberos-iv", 750, "udp");
+    add_port_service(family, "kerberos-iv", 750, "tcp");
     if(enable_http)
-	add_port(family, "http", "tcp");
+	add_port_service(family, "http", 80, "tcp");
 #ifdef KASERVER
-    add_port(family, "7004", "udp");
+    if (enable_kaserver)
+	add_port_service(family, "afs3-kaserver", 7004, "udp");
 #endif
 }
 
@@ -120,16 +137,16 @@ parse_ports(const char *str)
 	    if(q){
 		*q++ = 0;
 #ifdef HAVE_IPV6
-		add_port(AF_INET6, p, q);
+		add_port_string(AF_INET6, p, q);
 #endif
-		add_port(AF_INET, p, q);
+		add_port_string(AF_INET, p, q);
 	    }else {
 #ifdef HAVE_IPV6
-		add_port(AF_INET6, p, "udp");
-		add_port(AF_INET6, p, "tcp");
+		add_port_string(AF_INET6, p, "udp");
+		add_port_string(AF_INET6, p, "tcp");
 #endif
-		add_port(AF_INET, p, "udp");
-		add_port(AF_INET, p, "tcp");
+		add_port_string(AF_INET, p, "udp");
+		add_port_string(AF_INET, p, "tcp");
 	    }
 	}
 	    
@@ -155,37 +172,29 @@ static void
 init_socket(struct descr *d, krb5_address *a, int family, int type, int port)
 {
     krb5_error_code ret;
-    struct sockaddr *sa;
-    void *sa_buf;
+    struct sockaddr_storage __ss;
+    struct sockaddr *sa = (struct sockaddr *)&__ss;
     int sa_size;
 
     memset(d, 0, sizeof(*d));
     d->s = -1;
 
-    sa_size = krb5_max_sockaddr_size ();
-    sa_buf = malloc(sa_size);
-    if (sa_buf == NULL) {
-	kdc_log(0, "Failed to allocate %u bytes", sa_size);
-	return;
-    }
-    sa = (struct sockaddr *)sa_buf;
-
     ret = krb5_addr2sockaddr (a, sa, &sa_size, port);
     if (ret) {
 	krb5_warn(context, ret, "krb5_anyaddr");
 	close(d->s);
 	d->s = -1;
-	goto out;
+	return;
     }
 
     if (sa->sa_family != family)
-	goto out;
+	return;
 
     d->s = socket(family, type, 0);
     if(d->s < 0){
 	krb5_warn(context, errno, "socket(%d, %d, 0)", family, type);
 	d->s = -1;
-	goto out;
+	return;
     }
 #if defined(HAVE_SETSOCKOPT) && defined(SOL_SOCKET) && defined(SO_REUSEADDR)
     {
@@ -199,15 +208,13 @@ init_socket(struct descr *d, krb5_address *a, int family, int type, int port)
 	krb5_warn(context, errno, "bind(%d)", ntohs(port));
 	close(d->s);
 	d->s = -1;
-	goto out;
+	return;
     }
     if(type == SOCK_STREAM && listen(d->s, SOMAXCONN) < 0){
 	krb5_warn(context, errno, "listen");
 	close(d->s);
-	d->s = -1;
+	return;
     }
-out:
-    free (sa_buf);
 }
 
 /*
@@ -298,7 +305,7 @@ process_request(unsigned char *buf,
     }
 #endif
 #ifdef KASERVER
-    else {
+    else if (enable_kaserver) {
 	ret = do_kaserver (buf, len, reply, from, (struct sockaddr_in*)addr);
 	return ret;
     }
@@ -326,8 +333,8 @@ do_request(void *buf, size_t len, int sendlength,
 {
     krb5_error_code ret;
     krb5_data reply;
-    
     char addr[128];
+    
     addr_to_string(from, from_len, addr, sizeof(addr));
     
     reply.length = 0;
@@ -362,41 +369,30 @@ static void
 handle_udp(struct descr *d)
 {
     unsigned char *buf;
-    struct sockaddr *sa;
-    void *sa_buf;
-    int sa_size;
+    struct sockaddr_storage __ss;
+    struct sockaddr *sa = (struct sockaddr *)&__ss;
     int from_len;
     int n;
 
-    sa_size = krb5_max_sockaddr_size ();
-    sa_buf = malloc(sa_size);
-    if (sa_buf == NULL) {
-	kdc_log(0, "Failed to allocate %u bytes", sa_size);
-	return;
-    }
-    sa = (struct sockaddr *)sa_buf;
-    
     buf = malloc(max_request);
     if(buf == NULL){
 	kdc_log(0, "Failed to allocate %u bytes", max_request);
-	free (sa_buf);
 	return;
     }
 
-    from_len = sa_size;
+    from_len = sizeof(__ss);
     n = recvfrom(d->s, buf, max_request, 0, 
 		 sa, &from_len);
     if(n < 0){
 	krb5_warn(context, errno, "recvfrom");
 	goto out;
     }
-    if(n == 0){
+    if(n == 0) {
 	goto out;
     }
     do_request(buf, n, 0, d->s, sa, from_len);
 out:
     free (buf);
-    free (sa_buf);
 }
 
 static void
@@ -410,152 +406,256 @@ clear_descr(struct descr *d)
     d->s = -1;
 }
 
+
+/* remove HTTP %-quoting from buf */
+static int
+de_http(char *buf)
+{
+    char *p, *q;
+    for(p = q = buf; *p; p++, q++) {
+	if(*p == '%') {
+	    unsigned int x;
+	    if(sscanf(p + 1, "%2x", &x) != 1)
+		return -1;
+	    *q = x;
+	    p += 2;
+	} else
+	    *q = *p;
+    }
+    *q = '\0';
+    return 0;
+}
+
 #define TCP_TIMEOUT 4
 
+/*
+ * accept a new TCP connection on `d[index]'
+ */
+
+static void
+add_new_tcp (struct descr *d, int index, int min_free)
+{
+    struct sockaddr_storage __ss;
+    struct sockaddr *sa = (struct sockaddr *)&__ss;
+    int s;
+    int from_len;
+
+    from_len = sizeof(__ss);
+    s = accept(d[index].s, sa, &from_len);
+    if(s < 0){
+	krb5_warn(context, errno, "accept");
+	return;
+    }
+    if(min_free == -1){
+	close(s);
+	return;
+    }
+	    
+    d[min_free].s = s;
+    d[min_free].timeout = time(NULL) + TCP_TIMEOUT;
+    d[min_free].type = SOCK_STREAM;
+}
+
+/*
+ * Grow `d' to handle at least `n'.
+ * Return != 0 if fails
+ */
+
+static int
+grow_descr (struct descr *d, size_t n)
+{
+    if (d->size - d->len < n) {
+	unsigned char *tmp;
+
+	d->size += max(1024, d->len + n);
+	if (d->size >= max_request) {
+	    kdc_log(0, "Request exceeds max request size (%u bytes).",
+		    d->size);
+	    clear_descr(d);
+	    return -1;
+	}
+	tmp = realloc (d->buf, d->size);
+	if (tmp == NULL) {
+	    kdc_log(0, "Failed to re-allocate %u bytes.", d->size);
+	    clear_descr(d);
+	    return -1;
+	}
+	d->buf = tmp;
+    }
+    return 0;
+}
+
+/*
+ * Try to handle the TCP data at `d->buf, d->len'.
+ * Return -1 if failed, 0 if succesful, and 1 if data is complete.
+ */
+
+static int
+handle_vanilla_tcp (struct descr *d)
+{
+    krb5_storage *sp;
+    int32_t len;
+
+    sp = krb5_storage_from_mem(d->buf, d->len);
+    if (sp == NULL) {
+	kdc_log (0, "krb5_storage_from_mem failed");
+	return -1;
+    }
+    krb5_ret_int32(sp, &len);
+    krb5_storage_free(sp);
+    if(d->len - 4 >= len) {
+	memcpy(d->buf, d->buf + 4, d->len - 4);
+	return 1;
+    }
+    return 0;
+}
+
+/*
+ * Try to handle the TCP/HTTP data at `d->buf, d->len'.
+ * Return -1 if failed, 0 if succesful, and 1 if data is complete.
+ */
+
+static int
+handle_http_tcp (struct descr *d, const char *addr)
+{
+    char *s, *p, *t;
+    void *data;
+    char *proto;
+    int len;
+
+    s = (char *)d->buf;
+
+    p = strstr(s, "\r\n");
+    if (p == NULL) {
+	kdc_log(0, "Malformed HTTP request from %s", addr);
+	return -1;
+    }
+    *p = 0;
+
+    p = NULL;
+    t = strtok_r(s, " \t", &p);
+    if (t == NULL) {
+	kdc_log(0, "Malformed HTTP request from %s", addr);
+	return -1;
+    }
+    t = strtok_r(NULL, " \t", &p);
+    if(t == NULL) {
+	kdc_log(0, "Malformed HTTP request from %s", addr);
+	return -1;
+    }
+    data = malloc(strlen(t));
+    if (data == NULL) {
+	kdc_log(0, "Failed to allocate %u bytes", strlen(t));
+	return -1;
+    }
+    if(*t == '/')
+	t++;
+    if(de_http(t) != 0) {
+	kdc_log(0, "Malformed HTTP request from %s", addr);
+	kdc_log(5, "Request: %s", t);
+	free(data);
+	return -1;
+    }
+    proto = strtok_r(NULL, " \t", &p);
+    if (proto == NULL) {
+	kdc_log(0, "Malformed HTTP request from %s", addr);
+	free(data);
+	return -1;
+    }
+    len = base64_decode(t, data);
+    if(len <= 0){
+	const char *msg = 
+	    " 404 Not found\r\n"
+	    "Server: Heimdal/" VERSION "\r\n"
+	    "Content-type: text/html\r\n"
+	    "Content-transfer-encoding: 8bit\r\n\r\n"
+	    "<TITLE>404 Not found</TITLE>\r\n"
+	    "<H1>404 Not found</H1>\r\n"
+	    "That page doesn't exist, maybe you are looking for "
+	    "<A HREF=\"http://www.pdc.kth.se/heimdal\">Heimdal</A>?\r\n";
+	write(d->s, proto, strlen(proto));
+	write(d->s, msg, strlen(msg));
+	kdc_log(0, "HTTP request from %s is non KDC request", addr);
+	kdc_log(5, "Request: %s", t);
+	free(data);
+	return -1;
+    }
+    {
+	const char *msg = 
+	    " 200 OK\r\n"
+	    "Server: Heimdal/" VERSION "\r\n"
+	    "Content-type: application/octet-stream\r\n"
+	    "Content-transfer-encoding: binary\r\n\r\n";
+	write(d->s, proto, strlen(proto));
+	write(d->s, msg, strlen(msg));
+    }
+    memcpy(d->buf, data, len);
+    d->len = len;
+    free(data);
+    return 1;
+}
+
+/*
+ * Handle incoming data to the TCP socket in `d[index]'
+ */
+
 static void
 handle_tcp(struct descr *d, int index, int min_free)
 {
     unsigned char buf[1024];
     char addr[32];
-    void *sa_buf;
-    struct sockaddr *sa;
-    int sa_size;
+    struct sockaddr_storage __ss;
+    struct sockaddr *sa = (struct sockaddr *)&__ss;
     int from_len;
     int n;
+    int ret = 0;
 
-    sa_size = krb5_max_sockaddr_size ();
-    sa_buf = malloc(sa_size);
-    if (sa_buf == NULL) {
-	kdc_log(0, "Failed to allocate %u bytes", sa_size);
+    if (d[index].timeout == 0) {
+	add_new_tcp (d, index, min_free);
 	return;
     }
-    sa = (struct sockaddr *)sa_buf;
 
-    if(d[index].timeout == 0){
-	int s;
+    /*
+     * We can't trust recvfrom to return an address so we always call
+     * getpeername.
+     */
 
-	from_len = sa_size;
-	s = accept(d[index].s, sa, &from_len);
-	if(s < 0){
-	    krb5_warn(context, errno, "accept");
-	    goto out;
-	}
-	if(min_free == -1){
-	    close(s);
-	    goto out;
-	}
-	    
-	d[min_free].s = s;
-	d[min_free].timeout = time(NULL) + TCP_TIMEOUT;
-	d[min_free].type = SOCK_STREAM;
-	goto out;
-    }
-    from_len = sa_size;
-    n = recvfrom(d[index].s, buf, sizeof(buf), 0, 
-		 sa, &from_len);
+    n = recvfrom(d[index].s, buf, sizeof(buf), 0, NULL, NULL);
     if(n < 0){
 	krb5_warn(context, errno, "recvfrom");
-	goto out;
+	return;
     }
-    /* sometimes recvfrom doesn't return an address */
-    if(from_len == 0){
-	from_len = sa_size;
-	getpeername(d[index].s, sa, &from_len);
+    from_len = sizeof(__ss);
+    if (getpeername(d[index].s, sa, &from_len) < 0) {
+	krb5_warn(context, errno, "getpeername");
+	return;
     }
     addr_to_string(sa, from_len, addr, sizeof(addr));
-    if(d[index].size - d[index].len < n){
-	unsigned char *tmp;
-	d[index].size += 1024;
-	if(d[index].size >= max_request){
-	    kdc_log(0, "Request exceeds max request size (%u bytes).",
-		    d[index].size);
-	    clear_descr(d + index);
-	    goto out;
-	}
-	tmp = realloc(d[index].buf, d[index].size);
-	if(tmp == NULL){
-	    kdc_log(0, "Failed to re-allocate %u bytes.", d[index].size);
-	    clear_descr(d + index);
-	    goto out;
-	}
-	d[index].buf = tmp;
-    }
+    if (grow_descr (&d[index], n))
+	return;
     memcpy(d[index].buf + d[index].len, buf, n);
     d[index].len += n;
-    if(d[index].len > 4 && d[index].buf[0] == 0){
-	krb5_storage *sp;
-	int32_t len;
-	sp = krb5_storage_from_mem(d[index].buf, d[index].len);
-	krb5_ret_int32(sp, &len);
-	krb5_storage_free(sp);
-	if(d[index].len - 4 >= len){
-	    memcpy(d[index].buf, d[index].buf + 4, d[index].len - 4);
-	    n = 0;
-	}
+    if(d[index].len > 4 && d[index].buf[0] == 0) {
+	ret = handle_vanilla_tcp (&d[index]);
+    } else if(enable_http &&
+	      d[index].len >= 4 &&
+	      strncmp((char *)d[index].buf, "GET ", 4) == 0 && 
+	      strncmp((char *)d[index].buf + d[index].len - 4,
+		      "\r\n\r\n", 4) == 0) {
+	ret = handle_http_tcp (&d[index], addr);
+	if (ret < 0)
+	    clear_descr (d + index);
+    } else if (d[index].len > 4) {
+	kdc_log (0, "TCP data of strange type from %s", addr);
+	return;
     }
-    else if(enable_http &&
-	    strncmp((char *)d[index].buf, "GET ", 4) == 0 && 
-	    strncmp((char *)d[index].buf + d[index].len - 4,
-		    "\r\n\r\n", 4) == 0){
-	char *s, *p, *t;
-	void *data;
-	int len;
-	s = (char *)d[index].buf;
-	p = strstr(s, "\r\n");
-	*p = 0;
-	p = NULL;
-	strtok_r(s, " \t", &p);
-	t = strtok_r(NULL, " \t", &p);
-	if(t == NULL){
-	    kdc_log(0, "Malformed HTTP request from %s", addr);
-	    clear_descr(d + index);
-	    goto out;
-	}
-	data = malloc(strlen(t));
-	if (data == NULL) {
-	    kdc_log(0, "Failed to allocate %u bytes", strlen(t));
-	    goto out;
-	}
-	if(*t == '/')
-	    t++;
-	len = base64_decode(t, data);
-	if(len <= 0){
-	    const char *msg = 
-		"HTTP/1.1 404 Not found\r\n"
-		"Server: Heimdal/" VERSION "\r\n"
-		"Content-type: text/html\r\n"
-		"Content-transfer-encoding: 8bit\r\n\r\n"
-		"<TITLE>404 Not found</TITLE>\r\n"
-		"<H1>404 Not found</H1>\r\n"
-		"That page doesn't exist, maybe you are looking for "
-		"<A HREF=\"http://www.pdc.kth.se/heimdal\">Heimdal</A>?\r\n";
-	    write(d[index].s, msg, strlen(msg));
-	    free(data);
-	    clear_descr(d + index);
-	    kdc_log(0, "HTTP request from %s is non KDC request", addr);
-	    goto out;
-	}
-	{
-	    const char *msg = 
-		"HTTP/1.1 200 OK\r\n"
-		"Server: Heimdal/" VERSION "\r\n"
-		"Content-type: application/octet-stream\r\n"
-		"Content-transfer-encoding: binary\r\n\r\n";
-	    write(d[index].s, msg, strlen(msg));
-	}
-	memcpy(d[index].buf, data, len);
-	d[index].len = len;
-	n = 0;
-	free(data);
-    }
-    if(n == 0){
+    if (ret < 0)
+	return;
+    else if (ret == 1) {
 	do_request(d[index].buf, d[index].len, 1,
 		   d[index].s, sa, from_len);
 	clear_descr(d + index);
     }
-out:
-    free (sa_buf);
-    return;
 }
 
 void
