@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,7 +38,7 @@
 
 #include "kdc_locl.h"
 
-RCSID("$Id: kerberos5.c,v 1.100 1999/02/11 21:03:01 joda Exp $");
+RCSID("$Id: kerberos5.c,v 1.106 1999/10/17 10:35:33 assar Exp $");
 
 #define MAX_TIME ((time_t)((1U << 31) - 1))
 
@@ -115,11 +115,23 @@ find_etype(hdb_entry *princ, unsigned *etypes, unsigned len,
 	   Key **key, int *index)
 {
     int i;
-    krb5_error_code ret = -1;
-    for(i = 0; i < len ; i++)
-	if((ret = hdb_enctype2key(context, princ, etypes[i], key)) == 0)
-	    break;
-    if(index) *index = i;
+    krb5_error_code ret = KRB5KDC_ERR_ETYPE_NOSUPP;
+
+    for(i = 0; i < len ; i++) {
+	krb5_error_code tmp;
+
+	tmp = hdb_enctype2key(context, princ, etypes[i], key);
+	if (tmp == 0) {
+	    if ((*key)->key.keyvalue.length != 0) {
+		ret = 0;
+		break;
+	    } else {
+		ret = KRB5KDC_ERR_NULL_KEY;
+	    }
+	}
+    }
+    if(index)
+	*index = i;
     return ret;
 }
 
@@ -138,9 +150,9 @@ find_keys(hdb_entry *client,
     if(client){
 	/* find client key */
 	ret = find_etype(client, etypes, num_etypes, ckey, &i);
-	if(ret){
+	if (ret) {
 	    kdc_log(0, "Client has no support for etypes");
-	    return KRB5KDC_ERR_ETYPE_NOSUPP;
+	    return ret;
 	}
 	*cetype = etypes[i];
     }
@@ -148,9 +160,9 @@ find_keys(hdb_entry *client,
     if(server){
 	/* find server key */
 	ret = find_etype(server, etypes, num_etypes, skey, NULL);
-	if(ret){
+	if (ret) {
 	    kdc_log(0, "Server has no support for etypes");
-	    return KRB5KDC_ERR_ETYPE_NOSUPP;
+	    return ret;
 	}
 	*setype = (*skey)->key.keytype;
     }
@@ -293,6 +305,10 @@ get_pa_etype_info(METHOD_DATA *md, hdb_entry *client)
     }
     len = length_ETYPE_INFO(&pa);
     buf = malloc(len);
+    if (buf) {
+	free_ETYPE_INFO(&pa);
+	return ret;
+    }
     ret = encode_ETYPE_INFO(buf + len - 1, len, &pa, &len);
     free_ETYPE_INFO(&pa);
     if(ret) {
@@ -495,10 +511,19 @@ as_rep(KDC_REQ *req,
 	    
 	    ret = hdb_enctype2key(context, client, enc_data.etype, &pa_key);
 	    if(ret){
+		char *estr;
 		e_text = "No key matches pa-data";
 		ret = KRB5KDC_ERR_PREAUTH_FAILED;
-		kdc_log(5, "No client key matching pa-data -- %s", 
-			client_name);
+		if(krb5_enctype_to_string(context, enc_data.etype, &estr))
+		    estr = NULL;
+		if(estr == NULL)
+		    kdc_log(5, "No client key matching pa-data (%d) -- %s", 
+			    enc_data.etype, client_name);
+		else
+		    kdc_log(5, "No client key matching pa-data (%s) -- %s", 
+			    estr, client_name);
+		free(estr);
+		    
 		free_EncryptedData(&enc_data);
 		continue;
 	    }
@@ -723,6 +748,14 @@ as_rep(KDC_REQ *req,
 	copy_HostAddresses(b->addresses, et.caddr);
     }
     
+    {
+	krb5_data empty_string;
+      
+	krb5_data_zero(&empty_string); 
+	et.transited.tr_type = DOMAIN_X500_COMPRESS;
+	et.transited.contents = empty_string;
+    }
+     
     copy_EncryptionKey(&et.key, &ek.key);
 
     /* The MIT ASN.1 library (obviously) doesn't tell lengths encoded
@@ -829,7 +862,7 @@ check_tgs_flags(KDC_REQ_BODY *b, EncTicketPart *tgt, EncTicketPart *et)
 	    kdc_log(0, "Bad request to validate ticket");
 	    return KRB5KDC_ERR_BADOPTION;
 	}
-	if(*tgt->starttime < kdc_time){
+	if(*tgt->starttime > kdc_time){
 	    kdc_log(0, "Early request to validate ticket");
 	    return KRB5KRB_AP_ERR_TKT_NYV;
 	}
@@ -940,7 +973,7 @@ fix_transited_encoding(TransitedEncoding *tr,
 	char **realms = NULL, **tmp;
 	int num_realms = 0;
 	int i;
-	if(tr->tr_type){
+	if(tr->tr_type && tr->contents.length != 0) {
 	    if(tr->tr_type != DOMAIN_X500_COMPRESS){
 		kdc_log(0, "Unknown transited type: %u", 
 			tr->tr_type);
@@ -1251,6 +1284,7 @@ tgs_rep2(KDC_REQ_BODY *b,
     krb5_auth_context ac = NULL;
     krb5_ticket *ticket = NULL;
     krb5_flags ap_req_options;
+    krb5_flags verify_ap_req_flags;
     const char *e_text = NULL;
     krb5_crypto crypto;
 
@@ -1316,11 +1350,17 @@ tgs_rep2(KDC_REQ_BODY *b,
 	goto out2;
     }
     
+    if (b->kdc_options.validate)
+	verify_ap_req_flags = KRB5_VERIFY_AP_REQ_IGNORE_INVALID;
+    else
+	verify_ap_req_flags = 0;
+
     ret = krb5_verify_ap_req(context,
 			     &ac,
 			     &ap_req,
 			     princ,
 			     &tkey->key,
+			     verify_ap_req_flags,
 			     &ap_req_options,
 			     &ticket);
 			     
@@ -1434,7 +1474,7 @@ tgs_rep2(KDC_REQ_BODY *b,
 		ret = KRB5KDC_ERR_ETYPE_NOSUPP; /* XXX */
 		goto out;
 	    }
-	    ret = krb5_decrypt_ticket(context, t, &tkey->key, &adtkt);
+	    ret = krb5_decrypt_ticket(context, t, &tkey->key, &adtkt, 0);
 
 	    if(ret)
 		goto out;
@@ -1461,7 +1501,7 @@ tgs_rep2(KDC_REQ_BODY *b,
 	    Realm req_rlm, new_rlm;
 	    if(loop++ < 2 && (req_rlm = is_krbtgt(&sp->name))){
 		new_rlm = find_rpath(req_rlm);
-		if(new_rlm){
+		if(new_rlm) {
 		    kdc_log(5, "krbtgt for realm %s not found, trying %s", 
 			    req_rlm, new_rlm);
 		    krb5_free_principal(context, sp);
