@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -37,11 +37,16 @@
  */
 
 #include "kpasswd_locl.h"
+RCSID("$Id: kpasswdd.c,v 1.40 1999/09/10 10:20:42 assar Exp $");
+
 #include <kadm5/admin.h>
-RCSID("$Id: kpasswdd.c,v 1.30 1999/04/01 18:26:45 joda Exp $");
+#ifdef HAVE_DLFCN_H
+#include <dlfcn.h>
+#endif
+
+#include <hdb.h>
 
 static krb5_context context;
-static void *kadm5_handle;
 static krb5_log_facility *log_facility;
 
 static sig_atomic_t exit_flag = 0;
@@ -191,14 +196,10 @@ reply_priv (krb5_auth_context auth_context,
     krb5_data_free (&krb_priv_data);
 }
 
-static char *
-passwd_quality_check (krb5_data *pwd)
-{
-    if (pwd->length < 6)
-	return "Password too short";
-    else
-	return NULL;
-}
+/*
+ * Change the password for `principal', sending the reply back on `s'
+ * (`sa', `sa_size') to `pwd_data'.
+ */
 
 static void
 change (krb5_auth_context auth_context,
@@ -209,23 +210,42 @@ change (krb5_auth_context auth_context,
 	krb5_data *pwd_data)
 {
     krb5_error_code ret;
-    char *c;
+    char *client;
     kadm5_principal_ent_rec ent;
     krb5_key_data *kd;
     krb5_salt salt;
     krb5_keyblock new_keyblock;
-    char *pwd_reason;
+    const char *pwd_reason;
     int unchanged;
+    kadm5_config_params conf;
+    void *kadm5_handle;
 
-    krb5_unparse_name (context, principal, &c);
+    memset (&conf, 0, sizeof(conf));
+    
+    krb5_unparse_name (context, principal, &client);
 
-    krb5_warnx (context, "Changing password for %s", c);
-    free (c);
+    ret = kadm5_init_with_password_ctx(context, 
+				       client,
+				       NULL,
+				       KADM5_ADMIN_SERVICE,
+				       &conf, 0, 0, 
+				       &kadm5_handle);
+    if (ret) {
+	free (client);
+	krb5_warn (context, ret, "kadm5_init_with_password_ctx");
+	reply_priv (auth_context, s, sa, sa_size, 2,
+		    "Internal error");
+	return;
+    }
 
-    pwd_reason = passwd_quality_check (pwd_data);
+    krb5_warnx (context, "Changing password for %s", client);
+    free (client);
+
+    pwd_reason = kadm5_check_password_quality (context, principal, pwd_data);
     if (pwd_reason != NULL ) {
-	krb5_warnx (context, pwd_reason);
+	krb5_warnx (context, "%s", pwd_reason);
 	reply_priv (auth_context, s, sa, sa_size, 4, pwd_reason);
+	kadm5_destroy (kadm5_handle);
 	return;
     }
 
@@ -236,7 +256,8 @@ change (krb5_auth_context auth_context,
     if (ret) {
 	krb5_warn (context, ret, "kadm5_get_principal");
 	reply_priv (auth_context, s, sa, sa_size, 2,
-		    "kadm5_get_principal failed");
+		    "Internal error");
+	kadm5_destroy (kadm5_handle);
 	return;
     }
 
@@ -248,7 +269,7 @@ change (krb5_auth_context auth_context,
 
     kd = &ent.key_data[0];
     
-    salt.salttype = KRB5_PW_SALT;
+    salt.salttype         = kd->key_data_type[1];
     salt.saltvalue.length = kd->key_data_length[1];
     salt.saltvalue.data   = kd->key_data_contents[1];
 
@@ -276,7 +297,7 @@ change (krb5_auth_context auth_context,
 	if (tmp == NULL) {
 	    krb5_warnx (context, "malloc: out of memory");
 	    reply_priv (auth_context, s, sa, sa_size, 2,
-			"malloc failed");
+			"Internal error");
 	    goto out;
 	}
 	memcpy (tmp, pwd_data->data, pwd_data->length);
@@ -290,18 +311,20 @@ change (krb5_auth_context auth_context,
 	if (ret) {
 	    krb5_warn (context, ret, "kadm5_s_chpass_principal");
 	    reply_priv (auth_context, s, sa, sa_size, 2,
-			"change failed");
+			"Internal error");
 	    goto out;
 	}
     }
-    reply_priv (auth_context, s, sa, sa_size, 0, "password changed");
+    reply_priv (auth_context, s, sa, sa_size, 0, "Password changed");
 out:
     kadm5_free_principal_ent (kadm5_handle, &ent);
+    kadm5_destroy (kadm5_handle);
 }
 
 static int
 verify (krb5_auth_context *auth_context,
 	krb5_principal server,
+	krb5_keytab keytab,
 	krb5_ticket **ticket,
 	krb5_data *out_data,
 	int s,
@@ -319,13 +342,14 @@ verify (krb5_auth_context *auth_context,
     pkt_ver = (msg[2] << 8) | (msg[3]);
     ap_req_len = (msg[4] << 8) | (msg[5]);
     if (pkt_len != len) {
-	krb5_warnx (context, "Strange len: %d != %d", pkt_len, len);
-	reply_error (server, s, sa, sa_size, 0, 1, "bad length");
+	krb5_warnx (context, "Strange len: %ld != %ld", 
+		    (long)pkt_len, (long)len);
+	reply_error (server, s, sa, sa_size, 0, 1, "Bad request");
 	return 1;
     }
     if (pkt_ver != 0x0001) {
 	krb5_warnx (context, "Bad version (%d)", pkt_ver);
-	reply_error (server, s, sa, sa_size, 0, 1, "bad version");
+	reply_error (server, s, sa, sa_size, 0, 1, "Wrong program version");
 	return 1;
     }
 
@@ -336,19 +360,26 @@ verify (krb5_auth_context *auth_context,
 		       auth_context,
 		       &ap_req_data,
 		       server,
-		       NULL,
+		       keytab,
 		       NULL,
 		       ticket);
     if (ret) {
-	krb5_warn (context, ret, "krb5_rd_req");
-	reply_error (server, s, sa, sa_size, ret, 3, "rd_req failed");
+	if(ret == KRB5_KT_NOTFOUND) {
+	    char *name;
+	    krb5_unparse_name(context, server, &name);
+	    krb5_warnx (context, "krb5_rd_req: %s (%s)", 
+			krb5_get_err_text(context, ret), name);
+	    free(name);
+	} else
+	    krb5_warn (context, ret, "krb5_rd_req");
+	reply_error (server, s, sa, sa_size, ret, 3, "Authentication failed");
 	return 1;
     }
 
     if (!(*ticket)->ticket.flags.initial) {
 	krb5_warnx (context, "initial flag not set");
 	reply_error (server, s, sa, sa_size, ret, 1,
-		     "initial flag not set");
+		     "Bad request");
 	goto out;
     }
     krb_priv_data.data   = msg + 6 + ap_req_len;
@@ -362,7 +393,7 @@ verify (krb5_auth_context *auth_context,
     
     if (ret) {
 	krb5_warn (context, ret, "krb5_rd_priv");
-	reply_error (server, s, sa, sa_size, ret, 3, "rd_priv failed");
+	reply_error (server, s, sa, sa_size, ret, 3, "Bad request");
 	goto out;
     }
     return 0;
@@ -373,6 +404,7 @@ out:
 
 static void
 process (krb5_principal server,
+	 krb5_keytab keytab,
 	 int s,
 	 krb5_address *this_addr,
 	 struct sockaddr *sa,
@@ -413,7 +445,7 @@ process (krb5_principal server,
 	goto out;
     }
 
-    if (verify (&auth_context, server, &ticket, &out_data,
+    if (verify (&auth_context, server, keytab, &ticket, &out_data,
 		s, sa, sa_size, msg, len) == 0) {
 	change (auth_context,
 		ticket->client,
@@ -430,7 +462,8 @@ out:
 }
 
 static int
-doit (int port)
+doit (krb5_keytab keytab,
+      int port)
 {
     krb5_error_code ret;
     krb5_principal server;
@@ -440,15 +473,8 @@ doit (int port)
     krb5_addresses addrs;
     unsigned n, i;
     fd_set real_fdset;
-    void *sa_buf;
-    int sa_max_size;
-    struct sockaddr *sa;
-
-    sa_max_size = krb5_max_sockaddr_size ();
-    sa_buf = malloc (sa_max_size);
-    if (sa_buf == NULL)
-	krb5_errx (context, 1, "out of memory");
-    sa = (struct sockaddr *)sa_buf;
+    struct sockaddr_storage __ss;
+    struct sockaddr *sa = (struct sockaddr *)&__ss;
 
     ret = krb5_get_default_realm (context, &realm);
     if (ret)
@@ -466,9 +492,9 @@ doit (int port)
 
     free (realm);
 
-    ret = krb5_get_all_client_addrs (context, &addrs);
+    ret = krb5_get_all_server_addrs (context, &addrs);
     if (ret)
-	krb5_err (context, 1, ret, "krb5_get_all_client_addrs");
+	krb5_err (context, 1, ret, "krb5_get_all_server_addrs");
 
     n = addrs.len;
 
@@ -482,11 +508,16 @@ doit (int port)
 
 	krb5_addr2sockaddr (&addrs.val[i], sa, &sa_size, port);
 
+	
 	sockets[i] = socket (sa->sa_family, SOCK_DGRAM, 0);
 	if (sockets[i] < 0)
 	    krb5_err (context, 1, errno, "socket");
-	if (bind (sockets[i], sa, sa_size) < 0)
-	    krb5_err (context, 1, errno, "bind");
+	if (bind (sockets[i], sa, sa_size) < 0) {
+	    char str[128];
+	    size_t len;
+	    ret = krb5_print_address (&addrs.val[i], str, sizeof(str), &len);
+	    krb5_err (context, 1, errno, "bind(%s)", str);
+	}
 	maxfd = max (maxfd, sockets[i]);
 	FD_SET(sockets[i], &real_fdset);
     }
@@ -505,7 +536,7 @@ doit (int port)
 	for (i = 0; i < n; ++i)
 	    if (FD_ISSET(sockets[i], &fdset)) {
 		u_char buf[BUFSIZ];
-		int addrlen = sa_max_size;
+		int addrlen = sizeof(__ss);
 
 		ret = recvfrom (sockets[i], buf, sizeof(buf), 0,
 				sa, &addrlen);
@@ -516,7 +547,7 @@ doit (int port)
 			krb5_err (context, 1, errno, "recvfrom");
 		}
 
-		process (server, sockets[i],
+		process (server, keytab, sockets[i],
 			 &addrs.val[i],
 			 sa, addrlen,
 			 buf, ret);
@@ -525,7 +556,6 @@ doit (int port)
     krb5_free_addresses (context, &addrs);
     krb5_free_principal (context, server);
     krb5_free_context (context);
-    free (sa_buf);
     return 0;
 }
 
@@ -535,9 +565,23 @@ sigterm(int sig)
     exit_flag = 1;
 }
 
+const char *check_library  = NULL;
+const char *check_function = NULL;
+char *keytab_str = "HDB:";
+char *realm_str;
 int version_flag;
 int help_flag;
+
 struct getargs args[] = {
+#ifdef HAVE_DLOPEN
+    { "check-library", 0, arg_string, &check_library, 
+      "library to load password check function from", "library" },
+    { "check-function", 0, arg_string, &check_function,
+      "password check function to load", "function" },
+#endif
+    { "keytab", 'k', arg_string, &keytab_str, 
+      "keytab to get authentication key from", "kspec" },
+    { "realm", 'r', arg_string, &realm_str, "default realm", "realm" },
     { "version", 0, arg_flag, &version_flag },
     { "help", 0, arg_flag, &help_flag }
 };
@@ -546,10 +590,9 @@ int num_args = sizeof(args) / sizeof(args[0]);
 int
 main (int argc, char **argv)
 {
-    krb5_error_code ret;
-    kadm5_config_params conf;
-
     int optind;
+    krb5_keytab keytab;
+    krb5_error_code ret;
     
     optind = krb5_program_setup(&context, argc, argv, args, num_args, NULL);
     
@@ -560,19 +603,21 @@ main (int argc, char **argv)
 	exit(0);
     }
 
+    if(realm_str)
+	krb5_set_default_realm(context, realm_str);
+    
     krb5_openlog (context, "kpasswdd", &log_facility);
     krb5_set_warn_dest(context, log_facility);
 
-    memset (&conf, 0, sizeof(conf));
+    ret = krb5_kt_register(context, &hdb_kt_ops);
+    if(ret)
+	krb5_err(context, 1, ret, "krb5_kt_register");
+
+    ret = krb5_kt_resolve(context, keytab_str, &keytab);
+    if(ret)
+	krb5_err(context, 1, ret, "%s", keytab_str);
     
-    ret = kadm5_init_with_password_ctx(context, 
-				       KADM5_ADMIN_SERVICE,
-				       NULL,
-				       KADM5_ADMIN_SERVICE,
-				       &conf, 0, 0, 
-				       &kadm5_handle);
-    if (ret)
-	krb5_err (context, 1, ret, "kadm5_init_with_password_ctx");
+    kadm5_setup_passwd_quality_check (context, check_library, check_function);
 
 #ifdef HAVE_SIGACTION
     {
@@ -588,5 +633,7 @@ main (int argc, char **argv)
     signal(SIGINT, sigterm);
 #endif
 
-    return doit (krb5_getportbyname (context, "kpasswd", "udp", KPASSWD_PORT));
+    return doit (keytab,
+		 krb5_getportbyname (context, "kpasswd", 
+				     "udp", KPASSWD_PORT));
 }
