@@ -38,17 +38,132 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: context.c,v 1.41 1999/04/01 18:23:06 joda Exp $");
+RCSID("$Id: context.c,v 1.48 1999/11/04 01:56:28 assar Exp $");
 
-int issuid(void); /* XXX */
+#define INIT_FIELD(C, T, E, D, F)					\
+    (C)->E = krb5_config_get_ ## T ## _default ((C), NULL, (D), 	\
+						"libdefaults", F, NULL)
+
+#ifdef KRB4
+extern krb5_kt_ops krb4_fkt_ops;
+#endif
+
+/*
+ * Set the list of etypes `ret_etypes' from the configuration variable
+ * `name'
+ */
+
+static krb5_error_code
+set_etypes (krb5_context context,
+	    const char *name,
+	    krb5_enctype **ret_enctypes)
+{
+    char **etypes_str;
+    krb5_enctype *etypes;
+
+    etypes_str = krb5_config_get_strings(context, NULL, "libdefaults", 
+					 name, NULL);
+    if(etypes_str){
+	int i, j, k;
+	for(i = 0; etypes_str[i]; i++);
+	etypes = malloc((i+1) * sizeof(*etypes));
+	if (etypes == NULL) {
+	    krb5_config_free_strings (etypes_str);
+	    return ENOMEM;
+	}
+	for(j = 0, k = 0; j < i; j++) {
+	    if(krb5_string_to_enctype(context, etypes_str[j], &etypes[k]) == 0)
+		k++;
+	}
+	etypes[k] = ETYPE_NULL;
+	krb5_config_free_strings(etypes_str);
+	*ret_enctypes = etypes;
+    }
+    return 0;
+}
+
+/*
+ * read variables from the configuration file and set in `context'
+ */
+
+static krb5_error_code
+init_context_from_config_file(krb5_context context)
+{
+    const char * tmp;
+    INIT_FIELD(context, time, max_skew, 5 * 60, "clockskew");
+    INIT_FIELD(context, time, kdc_timeout, 3, "kdc_timeout");
+    INIT_FIELD(context, int, max_retries, 3, "max_retries");
+
+    context->http_proxy = krb5_config_get_string(context, NULL, "libdefaults", 
+					   "http_proxy", NULL);
+
+    set_etypes (context, "default_etypes", &context->etypes);
+    set_etypes (context, "default_etypes_des", &context->etypes_des);
+
+    /* default keytab name */
+    context->default_keytab = krb5_config_get_string(context, NULL, 
+					       "libdefaults", 
+					       "default_keytab_name", 
+					       NULL);
+    if(context->default_keytab == NULL)
+	context->default_keytab = KEYTAB_DEFAULT;
+
+    context->time_fmt = krb5_config_get_string(context, NULL, "libdefaults", 
+					 "time_format", NULL);
+    if(context->time_fmt == NULL)
+	context->time_fmt = "%d-%b-%Y %H:%M:%S";
+    context->log_utc = krb5_config_get_bool(context, NULL, "libdefaults",
+					    "log_utc", NULL);
+
+    /* init dns-proxy slime */
+    tmp = krb5_config_get_string(context, NULL, "libdefaults", 
+				 "dns_proxy", NULL);
+    if(tmp) 
+	roken_gethostby_setup(context->http_proxy, tmp);
+    krb5_set_default_realm(context, NULL);
+
+    {
+	krb5_addresses addresses;
+	char **adr, **a;
+	adr = krb5_config_get_strings(context, NULL, 
+				      "libdefaults", 
+				      "extra_addresses", 
+				      NULL);
+	memset(&addresses, 0, sizeof(addresses));
+	for(a = adr; a && *a; a++) {
+	    krb5_parse_address(context, *a, &addresses);
+	    krb5_add_extra_addresses(context, &addresses);
+	    krb5_free_addresses(context, &addresses);
+	}
+	krb5_config_free_strings(adr);
+    }
+    
+    INIT_FIELD(context, bool, srv_lookup, TRUE, "srv_lookup");
+    INIT_FIELD(context, bool, srv_try_txt, FALSE, "srv_try_txt");
+    INIT_FIELD(context, bool, srv_try_rfc2052, TRUE, "srv_try_rfc2052");
+    INIT_FIELD(context, int, fcache_vno, 0, "fcache_version");
+
+    context->cc_ops       = NULL;
+    context->num_cc_ops	  = 0;
+    krb5_cc_register(context, &krb5_fcc_ops, TRUE);
+    krb5_cc_register(context, &krb5_mcc_ops, TRUE);
+
+    context->num_kt_types = 0;
+    context->kt_types     = NULL;
+    krb5_kt_register (context, &krb5_fkt_ops);
+    krb5_kt_register (context, &krb5_mkt_ops);
+#ifdef KRB4
+    krb5_kt_register (context, &krb4_fkt_ops);
+#endif
+    krb5_kt_register (context, &krb5_akf_ops);
+    return 0;
+}
 
 krb5_error_code
 krb5_init_context(krb5_context *context)
 {
     krb5_context p;
-    int val;
     const char *config_file = NULL;
-    const char * tmp;
     krb5_config_section *tmp_cf;
     krb5_error_code ret;
 
@@ -75,76 +190,9 @@ krb5_init_context(krb5_context *context)
 		    config_file); /* XXX */
 #endif
 
-    p->max_skew = 5 * 60;
-    val = krb5_config_get_time (p, NULL, "libdefaults", "clockskew", NULL);
-    if (val >= 0)
-	p->max_skew = val;
-
-    p->kdc_timeout = 3;
-    val = krb5_config_get_time (p, NULL, "libdefaults", "kdc_timeout", NULL);
-    if(val >= 0) 
-	p->kdc_timeout = val;
-
-    p->max_retries = 3;
-    val = krb5_config_get_int (p, NULL, "libdefaults", "max_retries", NULL);
-    if (val >= 0)
-	p->max_retries = val;
-
-    p->http_proxy = krb5_config_get_string(p, NULL, "libdefaults", 
-					   "http_proxy", NULL);
-
-    {
-	char **etypes;
-	etypes = krb5_config_get_strings(p, NULL, "libdefaults", 
-					 "default_etypes", NULL);
-	if(etypes){
-	    int i, j, k;
-	    for(i = 0; etypes[i]; i++);
-	    p->etypes = malloc((i+1) * sizeof(*p->etypes));
-	    for(j = 0, k = 0; j < i; j++) {
-		if(krb5_string_to_enctype(p, etypes[j], &p->etypes[k]) == 0)
-		    k++;
-	    }
-	    p->etypes[k] = ETYPE_NULL;
-	    krb5_config_free_strings(etypes);
-	}
-    }
-    /* default keytab name */
-    p->default_keytab = krb5_config_get_string(p, NULL, 
-					       "libdefaults", 
-					       "default_keytab_name", 
-					       NULL);
-    if(p->default_keytab == NULL)
-	p->default_keytab = KEYTAB_DEFAULT;
-
-    p->time_fmt = krb5_config_get_string(p, NULL, "libdefaults", 
-					 "time_format", NULL);
-    if(p->time_fmt == NULL)
-	p->time_fmt = "%d-%b-%Y %H:%M:%S";
-    p->log_utc = krb5_config_get_bool(p, NULL, "libdefaults", "log_utc", NULL);
-
-    /* init dns-proxy slime */
-    tmp = krb5_config_get_string(p, NULL, "libdefaults", 
-				 "dns_proxy", NULL);
-    if(tmp) 
-	roken_gethostby_setup(p->http_proxy, tmp);
-    krb5_set_default_realm(p, NULL);
-
-    {
-	krb5_addresses addresses;
-	char **adr, **a;
-	adr = krb5_config_get_strings(p, NULL, 
-				      "libdefaults", 
-				      "extra_addresses", 
-				      NULL);
-	memset(&addresses, 0, sizeof(addresses));
-	for(a = adr; a && *a; a++) {
-	    krb5_parse_address(p, *a, &addresses);
-	    krb5_add_extra_addresses(p, &addresses);
-	    krb5_free_addresses(p, &addresses);
-	}
-	krb5_config_free_strings(adr);
-    }
+    ret = init_context_from_config_file(p);
+    if(ret)
+	return ret;
 
     *context = p;
     return 0;
@@ -156,12 +204,14 @@ krb5_free_context(krb5_context context)
   int i;
 
   free(context->etypes);
-  free(context->default_realm);
-  krb5_config_file_free (context->cf);
+  free(context->etypes_des);
+  krb5_free_host_realm (context, context->default_realms);
+  krb5_config_file_free (context, context->cf);
   free_error_table (context->et_list);
-  for(i = 0; i < context->num_ops; ++i)
+  for(i = 0; i < context->num_cc_ops; ++i)
     free(context->cc_ops[i].prefix);
   free(context->cc_ops);
+  free(context->kt_types);
   free(context);
 }
 
@@ -208,7 +258,6 @@ krb5_set_default_in_tkt_etypes(krb5_context context,
 }
 
 
-
 krb5_error_code
 krb5_get_default_in_tkt_etypes(krb5_context context,
 			       krb5_enctype **etypes)
@@ -231,7 +280,7 @@ krb5_get_default_in_tkt_etypes(krb5_context context,
 }
 
 const char *
-krb5_get_err_text(krb5_context context, long code)
+krb5_get_err_text(krb5_context context, krb5_error_code code)
 {
     const char *p = com_right(context->et_list, code);
     if(p == NULL)
@@ -296,3 +345,17 @@ krb5_get_extra_addresses(krb5_context context, krb5_addresses *addresses)
     }
     return copy_HostAddresses(context->extra_addresses, addresses);
 }
+
+krb5_error_code
+krb5_set_fcache_version(krb5_context context, int version)
+{
+    context->fcache_vno = version;
+    return 0;
+}
+
+krb5_error_code
+krb5_get_fcache_version(krb5_context context, int *version)
+{
+    *version = context->fcache_vno;
+    return 0;
+}
