@@ -37,7 +37,7 @@
  */
 
 #include "krb5_locl.h"
-RCSID("$Id: crypto.c,v 1.9 1999/04/01 17:02:55 joda Exp $");
+RCSID("$Id: crypto.c,v 1.21 1999/11/04 01:56:50 assar Exp $");
 
 #undef CRYPTO_DEBUG
 #ifdef CRYPTO_DEBUG
@@ -65,10 +65,11 @@ struct krb5_crypto_data {
 #define CRYPTO_ETYPE(C) ((C)->et->type)
 
 /* bits for `flags' below */
-#define F_KEYED		1	/* checksum is keyed */
-#define F_CPROOF	2	/* checksum is collision proof */
-#define F_DERIVED	4	/* uses derived keys */
-#define F_VARIANT	8	/* uses `variant' keys (6.4.3) */
+#define F_KEYED		 1	/* checksum is keyed */
+#define F_CPROOF	 2	/* checksum is collision proof */
+#define F_DERIVED	 4	/* uses derived keys */
+#define F_VARIANT	 8	/* uses `variant' keys (6.4.3) */
+#define F_PSEUDO	16	/* not a real protocol type */
 
 struct salt_type {
     krb5_salttype type;
@@ -321,7 +322,7 @@ DES3_string_to_key(krb5_context context,
     
     len = password.length + salt.saltvalue.length;
     str = malloc(len);
-    if(str == NULL)
+    if(len != 0 && str == NULL)
 	return ENOMEM;
     memcpy(str, password.data, password.length);
     memcpy(str + password.length, salt.saltvalue.data, salt.saltvalue.length);
@@ -369,9 +370,10 @@ DES3_string_to_key_derived(krb5_context context,
 {
     krb5_error_code ret;
     size_t len = password.length + salt.saltvalue.length;
-    char *s = malloc(len);
+    char *s;
 
-    if(s == NULL)
+    s = malloc(len);
+    if(len != 0 && s == NULL)
 	return ENOMEM;
     memcpy(s, password.data, password.length);
     memcpy(s + password.length, salt.saltvalue.data, salt.saltvalue.length);
@@ -507,6 +509,27 @@ krb5_salttype_to_string (krb5_context context,
     return HEIM_ERR_SALTTYPE_NOSUPP;
 }
 
+krb5_error_code
+krb5_string_to_salttype (krb5_context context,
+			 krb5_enctype etype,
+			 const char *string,
+			 krb5_salttype *salttype)
+{
+    struct encryption_type *e;
+    struct salt_type *st;
+
+    e = _find_enctype (etype);
+    if (e == NULL)
+	return KRB5_PROG_ETYPE_NOSUPP;
+    for (st = e->keytype->string_to_key; st && st->type; st++) {
+	if (strcasecmp (st->name, string) == 0) {
+	    *salttype = st->type;
+	    return 0;
+	}
+    }
+    return HEIM_ERR_SALTTYPE_NOSUPP;
+}
+
 krb5_error_code
 krb5_get_pw_salt(krb5_context context,
 		 krb5_const_principal principal,
@@ -656,7 +679,7 @@ krb5_generate_random_keyblock(krb5_context context,
 
 static krb5_error_code
 _key_schedule(krb5_context context,
-	     struct key_data *key)
+	      struct key_data *key)
 {
     krb5_error_code ret;
     struct encryption_type *et = _find_enctype(key->key->keytype);
@@ -971,13 +994,8 @@ HMAC_SHA1_DES3_checksum(krb5_context context,
 			Checksum *result)
 {
     struct checksum_type *c = _find_checksum(CKSUMTYPE_SHA1);
-    /* iovec? */
-    unsigned char *p = malloc(4 + len);
-    _krb5_put_int(p, len, 4);
-    memcpy(p + 4, data , len);
-    hmac(context, c, p, 4 + len, key, result);
-    memset(p, 0, 4 + len);
-    free(p);
+
+    hmac(context, c, data, len, key, result);
 }
 
 struct checksum_type checksum_none = {
@@ -1073,7 +1091,7 @@ struct checksum_type checksum_rsa_md5_des3 = {
 struct checksum_type checksum_sha1 = {
     CKSUMTYPE_SHA1,
     "sha1",
-    80,
+    64,
     20,
     F_CPROOF,
     SHA1_checksum,
@@ -1082,7 +1100,7 @@ struct checksum_type checksum_sha1 = {
 struct checksum_type checksum_hmac_sha1_des3 = {
     CKSUMTYPE_HMAC_SHA1_DES3,
     "hmac-sha1-des3",
-    80,
+    64,
     20,
     F_KEYED | F_CPROOF | F_DERIVED,
     HMAC_SHA1_DES3_checksum,
@@ -1126,21 +1144,22 @@ get_checksum_key(krb5_context context,
 		 struct key_data **key)
 {
     krb5_error_code ret = 0;
+
     if(ct->flags & F_DERIVED)
 	ret = _get_derived_key(context, crypto, usage, key);
-    else {
-	if(ct->flags & F_VARIANT) {
-	    int i;
-	    *key = _new_derived_key(crypto, 0xff/* KRB5_KU_RFC1510_VARIANT */);
-	    if(*key == NULL)
-		return ENOMEM;
-	    ret = krb5_copy_keyblock(context, crypto->key.key, &(*key)->key);
-	    if(ret) 
-		return ret;
-	    for(i = 0; i < (*key)->key->keyvalue.length; i++)
-		((unsigned char*)(*key)->key->keyvalue.data)[i] ^= 0xF0;
-	} else
-	    *key = &crypto->key; 
+    else if(ct->flags & F_VARIANT) {
+	int i;
+
+	*key = _new_derived_key(crypto, 0xff/* KRB5_KU_RFC1510_VARIANT */);
+	if(*key == NULL)
+	    return ENOMEM;
+	ret = krb5_copy_keyblock(context, crypto->key.key, &(*key)->key);
+	if(ret) 
+	    return ret;
+	for(i = 0; i < (*key)->key->keyvalue.length; i++)
+	    ((unsigned char*)(*key)->key->keyvalue.data)[i] ^= 0xF0;
+    } else {
+	*key = &crypto->key; 
     }
     if(ret == 0)
 	ret = _key_schedule(context, *key);
@@ -1148,26 +1167,18 @@ get_checksum_key(krb5_context context,
 }
 
 static krb5_error_code
-create_checksum(krb5_context context,
-		krb5_crypto crypto,
-		unsigned usage, /* not krb5_key_usage */
-		krb5_cksumtype type, /* if crypto == NULL */
-		void *data,
-		size_t len,
-		Checksum *result)
+do_checksum (krb5_context context,
+	     struct checksum_type *ct,
+	     krb5_crypto crypto,
+	     unsigned usage,
+	     void *data,
+	     size_t len,
+	     Checksum *result)
 {
     krb5_error_code ret;
-    struct checksum_type *ct;
     struct key_data *dkey;
     int keyed_checksum;
-    if(crypto) {
-	ct = crypto->et->keyed_checksum;
-	if(ct == NULL)
-	    ct = crypto->et->cksumtype;
-    } else
-	ct = _find_checksum(type);
-    if(ct == NULL)
-	return KRB5_PROG_SUMTYPE_NOSUPP;
+
     keyed_checksum = (ct->flags & F_KEYED) != 0;
     if(keyed_checksum && crypto == NULL)
 	return KRB5_PROG_SUMTYPE_NOSUPP; /* XXX */
@@ -1181,6 +1192,28 @@ create_checksum(krb5_context context,
     return 0;
 }
 
+static krb5_error_code
+create_checksum(krb5_context context,
+		krb5_crypto crypto,
+		unsigned usage, /* not krb5_key_usage */
+		krb5_cksumtype type, /* if crypto == NULL */
+		void *data,
+		size_t len,
+		Checksum *result)
+{
+    struct checksum_type *ct;
+
+    if(crypto) {
+	ct = crypto->et->keyed_checksum;
+	if(ct == NULL)
+	    ct = crypto->et->cksumtype;
+    } else
+	ct = _find_checksum(type);
+    if(ct == NULL)
+	return KRB5_PROG_SUMTYPE_NOSUPP;
+    return do_checksum (context, ct, crypto, usage, data, len, result);
+}
+
 krb5_error_code
 krb5_create_checksum(krb5_context context,
 		     krb5_crypto crypto,
@@ -1207,12 +1240,8 @@ verify_checksum(krb5_context context,
     int keyed_checksum;
     Checksum c;
     struct checksum_type *ct;
-    if(crypto) {
-	ct = crypto->et->keyed_checksum;
-	if(ct == NULL)
-	    ct = crypto->et->cksumtype;
-    } else
-	ct = _find_checksum(cksum->cksumtype);
+
+    ct = _find_checksum(cksum->cksumtype);
     if(ct == NULL)
 	return KRB5_PROG_SUMTYPE_NOSUPP;
     if(ct->checksumsize != cksum->checksum.length)
@@ -1227,16 +1256,18 @@ verify_checksum(krb5_context context,
     if(ct->verify)
 	return (*ct->verify)(context, dkey, data, len, cksum);
 
-    ret = create_checksum(context, crypto, usage, ct->type, data, len, &c);
-    if(ret)
+    ret = krb5_data_alloc (&c.checksum, ct->checksumsize);
+    if (ret)
 	return ret;
-    
+
+    (*ct->checksum)(context, dkey, data, len, &c);
+
     if(c.checksum.length != cksum->checksum.length || 
        memcmp(c.checksum.data, cksum->checksum.data, c.checksum.length))
 	ret = KRB5KRB_AP_ERR_BAD_INTEGRITY;
     else
 	ret = 0;
-    free_Checksum(&c);
+    krb5_data_free (&c.checksum);
     return ret;
 }
 
@@ -1332,6 +1363,10 @@ DES3_CBC_encrypt(struct key_data *key,
     des_ede3_cbc_encrypt(data, data, len, s[0], s[1], s[2], &ivec, encrypt);
 }
 
+/*
+ * these should currently be in reverse preference order.
+ */
+
 static struct encryption_type etypes[] = {
     {
 	ETYPE_NULL,
@@ -1388,7 +1423,6 @@ static struct encryption_type etypes[] = {
 	0,
  	DES3_CBC_encrypt,
     },
-#if NEW_DES3_CODE
     {
 	ETYPE_DES3_CBC_SHA1,
 	"des3-cbc-sha1",
@@ -1400,21 +1434,9 @@ static struct encryption_type etypes[] = {
 	F_DERIVED,
  	DES3_CBC_encrypt,
     },
-#else
     {
-	ETYPE_NEW_DES3_CBC_SHA1,
-	"new-des3-cbc-sha1",
-	8,
-	8,
-	&keytype_des3_derived,
-	&checksum_sha1,
-	&checksum_hmac_sha1_des3,
-	F_DERIVED,
- 	DES3_CBC_encrypt,
-    },
-    {
-	ETYPE_DES3_CBC_SHA1,
-	"des3-cbc-sha1",
+	ETYPE_OLD_DES3_CBC_SHA1,
+	"old-des3-cbc-sha1",
 	8,
 	8,
 	&keytype_des3,
@@ -1423,7 +1445,6 @@ static struct encryption_type etypes[] = {
 	0,
  	DES3_CBC_encrypt,
     },
-#endif
     {
 	ETYPE_DES_CBC_NONE,
 	"des-cbc-none",
@@ -1432,7 +1453,7 @@ static struct encryption_type etypes[] = {
 	&keytype_des,
 	&checksum_none,
 	NULL,
-	0,
+	F_PSEUDO,
 	DES_CBC_encrypt_null_ivec,
     },
     {
@@ -1443,7 +1464,7 @@ static struct encryption_type etypes[] = {
 	&keytype_des3_derived,
 	&checksum_none,
 	NULL,
-	0,
+	F_PSEUDO,
 	DES_CBC_encrypt_null_ivec,
     },
 };
@@ -1497,7 +1518,6 @@ krb5_enctype_to_keytype(krb5_context context,
 			krb5_keytype *keytype)
 {
     struct encryption_type *e = _find_enctype(etype);
-    krb5_warnx(context, "krb5_enctype_to_keytype(%u)", etype);
     if(e == NULL)
 	return KRB5_PROG_ETYPE_NOSUPP;
     *keytype = e->keytype->type; /* XXX */
@@ -1519,6 +1539,63 @@ krb5_keytype_to_enctype(krb5_context context,
 }
 #endif
     
+krb5_error_code
+krb5_keytype_to_enctypes (krb5_context context,
+			  krb5_keytype keytype,
+			  unsigned *len,
+			  int **val)
+{
+    int i;
+    unsigned n = 0;
+    int *ret;
+
+    for (i = num_etypes - 1; i >= 0; --i) {
+	if (etypes[i].keytype->type == keytype
+	    && !(etypes[i].flags & F_PSEUDO))
+	    ++n;
+    }
+    ret = malloc(n * sizeof(int));
+    if (ret == NULL && n != 0)
+	return ENOMEM;
+    n = 0;
+    for (i = num_etypes - 1; i >= 0; --i) {
+	if (etypes[i].keytype->type == keytype
+	    && !(etypes[i].flags & F_PSEUDO))
+	    ret[n++] = etypes[i].type;
+    }
+    *len = n;
+    *val = ret;
+    return 0;
+}
+
+/*
+ * First take the configured list of etypes for `keytype' if available,
+ * else, do `krb5_keytype_to_enctypes'.
+ */
+
+krb5_error_code
+krb5_keytype_to_enctypes_default (krb5_context context,
+				  krb5_keytype keytype,
+				  unsigned *len,
+				  int **val)
+{
+    int i, n;
+    int *ret;
+
+    if (keytype != KEYTYPE_DES || context->etypes_des == NULL)
+	return krb5_keytype_to_enctypes (context, keytype, len, val);
+
+    for (n = 0; context->etypes_des[n]; ++n)
+	;
+    ret = malloc (n * sizeof(*ret));
+    if (ret == NULL && n != 0)
+	return ENOMEM;
+    for (i = 0; i < n; ++i)
+	ret[i] = context->etypes_des[i];
+    *len = n;
+    *val = ret;
+    return 0;
+}
 
 krb5_error_code
 krb5_enctype_valid(krb5_context context, 
@@ -1529,9 +1606,9 @@ krb5_enctype_valid(krb5_context context,
 
 /* if two enctypes have compatible keys */
 krb5_boolean
-krb5_enctypes_comptible_keys(krb5_context context,
-			     krb5_enctype etype1,
-			     krb5_enctype etype2)
+krb5_enctypes_compatible_keys(krb5_context context,
+			      krb5_enctype etype1,
+			      krb5_enctype etype2)
 {
     struct encryption_type *e1 = _find_enctype(etype1);
     struct encryption_type *e2 = _find_enctype(etype2);
@@ -1566,7 +1643,7 @@ encrypt_internal_derived(krb5_context context,
     
     checksum_sz = CHECKSUMSIZE(et->keyed_checksum);
 
-    sz = et->confoundersize + 4 /* length */ + len;
+    sz = et->confoundersize + /* 4 - length */ len;
     block_sz = (sz + et->blocksize - 1) &~ (et->blocksize - 1); /* pad */
     p = calloc(1, block_sz + checksum_sz);
     if(p == NULL)
@@ -1575,8 +1652,6 @@ encrypt_internal_derived(krb5_context context,
     q = p;
     krb5_generate_random_block(q, et->confoundersize); /* XXX */
     q += et->confoundersize;
-    _krb5_put_int(q, len, 4);
-    q += 4;
     memcpy(q, data, len);
     
     ret = create_checksum(context, 
@@ -1650,8 +1725,10 @@ encrypt_internal(krb5_context context,
 			  p, 
 			  block_sz,
 			  &cksum);
-    if(ret == 0 && cksum.checksum.length != checksum_sz)
+    if(ret == 0 && cksum.checksum.length != checksum_sz) {
+	free_Checksum (&cksum);
 	ret = KRB5_CRYPTO_INTERNAL;
+    }
     if(ret) {
 	memset(p, 0, block_sz);
 	free(p);
@@ -1692,7 +1769,7 @@ decrypt_internal_derived(krb5_context context,
     unsigned long l;
     
     p = malloc(len);
-    if(p == NULL)
+    if(len != 0 && p == NULL)
 	return ENOMEM;
     memcpy(p, data, len);
 
@@ -1714,8 +1791,9 @@ decrypt_internal_derived(krb5_context context,
 #endif
     (*et->encrypt)(dkey, p, len, 0);
 
-    cksum.checksum.data = p + len;
+    cksum.checksum.data   = p + len;
     cksum.checksum.length = checksum_sz;
+    cksum.cksumtype       = CHECKSUMTYPE(et->keyed_checksum);
 
     ret = verify_checksum(context,
 			  crypto,
@@ -1727,8 +1805,8 @@ decrypt_internal_derived(krb5_context context,
 	free(p);
 	return ret;
     }
-    _krb5_get_int(p + et->confoundersize, &l, 4);
-    memmove(p, p + et->confoundersize + 4, l);
+    l = len - et->confoundersize;
+    memmove(p, p + et->confoundersize, l);
     result->data = realloc(p, l);
     if(p == NULL) {
 	free(p);
@@ -1753,7 +1831,7 @@ decrypt_internal(krb5_context context,
     
     checksum_sz = CHECKSUMSIZE(et->cksumtype);
     p = malloc(len);
-    if(p == NULL)
+    if(len != 0 && p == NULL)
 	return ENOMEM;
     memcpy(p, data, len);
     
@@ -1879,9 +1957,46 @@ krb5_generate_random_block(void *buf, size_t len)
     }
 }
 
+static void
+DES3_postproc(krb5_context context,
+	      unsigned char *k, size_t len, struct key_data *key)
+{
+    unsigned char x[24];
+    int i, j;
+
+    memset(x, 0, sizeof(x));
+    for (i = 0; i < 3; ++i) {
+	unsigned char foo;
+
+	for (j = 0; j < 7; ++j) {
+	    unsigned char b = k[7 * i + j];
+
+	    x[8 * i + j] = b;
+	}
+	foo = 0;
+	for (j = 6; j >= 0; --j) {
+	    foo |= k[7 * i + j] & 1;
+	    foo <<= 1;
+	}
+	x[8 * i + 7] = foo;
+    }
+    k = key->key->keyvalue.data;
+    memcpy(k, x, 24);
+    memset(x, 0, sizeof(x));
+    if (key->schedule) {
+	krb5_free_data(context, key->schedule);
+	key->schedule = NULL;
+    }
+    des_set_odd_parity((des_cblock*)k);
+    des_set_odd_parity((des_cblock*)(k + 8));
+    des_set_odd_parity((des_cblock*)(k + 16));
+}
+
+#if 0
 /* XXX should be moved someplace else */
 static void
-DES3_postproc(unsigned char *k, size_t len, struct key_data *key)
+DES3_postproc(krb5_context context,
+	      unsigned char *k, size_t len, struct key_data *key)
 {
     unsigned char x[24];
     int ki = 0, xi = 0, kb = 8, xb = 8;
@@ -1911,13 +2026,15 @@ DES3_postproc(unsigned char *k, size_t len, struct key_data *key)
     k = key->key->keyvalue.data;
     memcpy(k, x, 24);
     memset(x, 0, sizeof(x));
-    krb5_data_free(key->schedule);
-    free(key->schedule);
-    key->schedule = NULL;
+    if (key->schedule) {
+	krb5_free_data(context, key->schedule);
+	key->schedule = NULL;
+    }
     des_set_odd_parity((des_cblock*)k);
     des_set_odd_parity((des_cblock*)(k + 8));
     des_set_odd_parity((des_cblock*)(k + 16));
 }
+#endif
 
 static krb5_error_code
 derive_key(krb5_context context,
@@ -1950,21 +2067,23 @@ derive_key(krb5_context context,
 	}
     } else {
 	void *c = malloc(len);
-	if(c == NULL)
+	size_t res_len = (kt->bits + 7) / 8;
+
+	if(len != 0 && c == NULL)
 	    return ENOMEM;
 	memcpy(c, constant, len);
 	(*et->encrypt)(key, c, len, 1);
-	k = malloc((kt->bits + 7) / 8);
-	if(k == NULL)
+	k = malloc(res_len);
+	if(res_len != 0 && k == NULL)
 	    return ENOMEM;
-	_krb5_n_fold(c, len, k, kt->bits);
+	_krb5_n_fold(c, len, k, res_len);
 	free(c);
     }
     
     /* XXX keytype dependent post-processing */
     switch(kt->type) {
     case KEYTYPE_DES3:
-	DES3_postproc(k, nblocks * et->blocksize, key);
+	DES3_postproc(context, k, nblocks * et->blocksize, key);
 	break;
     default:
 	krb5_warnx(context, "derive_key() called with unknown keytype (%u)", 
@@ -1999,7 +2118,8 @@ _get_derived_key(krb5_context context,
 {
     int i;
     struct key_data *d;
-    unsigned char constant[4];
+    unsigned char constant[5];
+
     for(i = 0; i < crypto->num_key_usage; i++)
 	if(crypto->key_usage[i].usage == usage) {
 	    *key = &crypto->key_usage[i].key;
@@ -2009,7 +2129,7 @@ _get_derived_key(krb5_context context,
     if(d == NULL)
 	return ENOMEM;
     krb5_copy_keyblock(context, crypto->key.key, &d->key);
-    _krb5_put_int(constant, usage, 4);
+    _krb5_put_int(constant, usage, 5);
     derive_key(context, crypto->et, d, constant, sizeof(constant));
     *key = d;
     return 0;
@@ -2084,13 +2204,17 @@ krb5_string_to_key_derived(krb5_context context,
     struct encryption_type *et = _find_enctype(etype);
     krb5_error_code ret;
     struct key_data kd;
+    u_char *tmp;
 
     if(et == NULL)
 	return KRB5_PROG_ETYPE_NOSUPP;
     ALLOC(kd.key, 1);
     kd.key->keytype = etype;
+    tmp = malloc (et->keytype->bits / 8);
+    _krb5_n_fold(str, len, tmp, et->keytype->bits / 8);
     krb5_data_alloc(&kd.key->keyvalue, et->keytype->size);
-    _krb5_n_fold(str, len, kd.key->keyvalue.data, kd.key->keyvalue.length);
+    kd.schedule = NULL;
+    DES3_postproc (context, tmp, et->keytype->bits / 8, &kd); /* XXX */
     ret = derive_key(context,
 		     et,
 		     &kd,
@@ -2101,6 +2225,24 @@ krb5_string_to_key_derived(krb5_context context,
     return ret;
 }
 
+/*
+ * Return the size of an encrypted packet of length `data_len'
+ */
+
+size_t
+krb5_get_wrapped_length (krb5_context context,
+			 krb5_crypto  crypto,
+			 size_t       data_len)
+{
+    struct encryption_type *et = crypto->et;
+    size_t blocksize = et->blocksize;
+    size_t res;
+
+    res = (data_len + blocksize - 1) / blocksize * blocksize;
+    res = res + et->confoundersize + et->cksumtype->checksumsize;
+    return res;
+}
+
 #ifdef CRYPTO_DEBUG
 
 static krb5_error_code
