@@ -38,7 +38,7 @@
 
 #include <krb5_locl.h>
 
-RCSID("$Id: get_cred.c,v 1.69 1999/04/01 18:24:35 joda Exp $");
+RCSID("$Id: get_cred.c,v 1.74 1999/11/04 01:57:17 assar Exp $");
 
 /*
  * Take the `body' and encode it into `padata' using the credentials
@@ -85,30 +85,9 @@ make_pa_tgs_req(krb5_context context,
 
     in_data.length = len;
     in_data.data   = buf + buf_size - len;
-    {
-	Ticket ticket;
-	ret = decode_Ticket(creds->ticket.data, creds->ticket.length, 
-			    &ticket, &len);
-	if(ret)
-	    return ret;
-	/*
-	 * If we get a ticket encrypted with DES-CBC-CRC, it's
-	 * probably an old DCE secd and then the usual heuristics of
-	 * using the best algorithm (in this case RSA-MD5 and
-	 * DES-CBC-MD5) will not work.
-	 */
-	if(ticket.enc_part.etype == ETYPE_DES_CBC_CRC) {
-	    krb5_auth_setcksumtype(context, ac, CKSUMTYPE_RSA_MD4);
-	    krb5_auth_setenctype(context, ac, ETYPE_DES_CBC_CRC);
-	}
-	free_Ticket(&ticket);
-	    
-	
-	ret = krb5_mk_req_internal(context, &ac, 0, &in_data, creds, 
-				   &padata->padata_value,
-				   KRB5_KU_TGS_REQ_AUTH_CKSUM);
-
-    }
+    ret = krb5_mk_req_internal(context, &ac, 0, &in_data, creds,
+			       &padata->padata_value,
+			       KRB5_KU_TGS_REQ_AUTH_CKSUM);
 out:
     free (buf);
     if(ret)
@@ -117,6 +96,60 @@ out:
     return 0;
 }
 
+/*
+ * Set the `enc-authorization-data' in `req_body' based on `authdata'
+ */
+
+static krb5_error_code
+set_auth_data (krb5_context context,
+	       KDC_REQ_BODY *req_body,
+	       krb5_authdata *authdata,
+	       krb5_keyblock *key)
+{
+    if(authdata->len) {
+	size_t len;
+	unsigned char *buf;
+	krb5_crypto crypto;
+	krb5_error_code ret;
+
+	len = length_AuthorizationData(authdata);
+	buf = malloc(len);
+	if (buf == NULL)
+	    return ENOMEM;
+	ret = encode_AuthorizationData(buf + len - 1,
+				       len, authdata, &len);
+	if (ret) {
+	    free (buf);
+	    return ret;
+	}
+
+	ALLOC(req_body->enc_authorization_data, 1);
+	if (req_body->enc_authorization_data == NULL) {
+	    free (buf);
+	    return ret;
+	}
+	ret = krb5_crypto_init(context, key, 0, &crypto);
+	if (ret) {
+	    free (buf);
+	    free (req_body->enc_authorization_data);
+	    return ret;
+	}
+	krb5_encrypt_EncryptedData(context, 
+				   crypto,
+				   KRB5_KU_TGS_REQ_AUTH_DAT_SUBKEY, 
+				   /* KRB5_KU_TGS_REQ_AUTH_DAT_SESSION? */
+				   buf,
+				   len,
+				   0,
+				   req_body->enc_authorization_data);
+	free (buf);
+	krb5_crypto_destroy(context, crypto);
+    } else {
+	req_body->enc_authorization_data = NULL;
+    }
+    return 0;
+}    
+
 /*
  * Create a tgs-req in `t' with `addresses', `flags', `second_ticket'
  * (if not-NULL), `in_creds', `krbtgt', and returning the generated
@@ -141,13 +174,10 @@ init_tgs_req (krb5_context context,
     t->pvno = 5;
     t->msg_type = krb_tgs_req;
     if (in_creds->session.keytype) {
-	krb5_enctype foo[2];
-	foo[0] = in_creds->session.keytype;
-	foo[1] = 0;
-	ret = krb5_init_etype(context,
-			      &t->req_body.etype.len,
-			      &t->req_body.etype.val,
-			      foo);
+	ret = krb5_keytype_to_enctypes_default (context,
+						in_creds->session.keytype,
+						&t->req_body.etype.len,
+						&t->req_body.etype.val);
     } else {
 	ret = krb5_init_etype(context, 
 			      &t->req_body.etype.len, 
@@ -209,43 +239,39 @@ init_tgs_req (krb5_context context,
     {
 	krb5_auth_context ac;
 	krb5_keyblock *key;
+
 	ret = krb5_auth_con_init(context, &ac);
 	if(ret)
-	    return ret;
+	    goto fail;
 	ret = krb5_generate_subkey (context, &krbtgt->session, &key);
+	if (ret) {
+	    krb5_auth_con_free (context, ac);
+	    goto fail;
+	}
 	ret = krb5_auth_con_setlocalsubkey(context, ac, key);
-	if(ret == ENOMEM)
-	    /* XXX */;
+	if (ret) {
+	    krb5_free_keyblock (context, key);
+	    krb5_auth_con_free (context, ac);
+	    goto fail;
+	}
 
-	if(in_creds->authdata.len){
-	    size_t len;
-	    unsigned char *buf;
-	    krb5_crypto crypto;
-	    len = length_AuthorizationData(&in_creds->authdata);
-	    buf = malloc(len);
-	    ret = encode_AuthorizationData(buf + len - 1, len, &in_creds->authdata, &len);
-	    ALLOC(t->req_body.enc_authorization_data, 1);
-	    ret = krb5_crypto_init(context, key, 0, &crypto);
-	    krb5_encrypt_EncryptedData(context, 
-				       crypto,
-				       KRB5_KU_TGS_REQ_AUTH_DAT_SUBKEY, 
-				       /* KRB5_KU_TGS_REQ_AUTH_DAT_SESSION? */
-				       buf, 
-				       len, 
-				       0,
-				       t->req_body.enc_authorization_data);
-	    krb5_crypto_destroy(context, crypto);
-	}else
-	    t->req_body.enc_authorization_data = NULL;
-    
+	ret = set_auth_data (context, &t->req_body, &in_creds->authdata, key);
+	if (ret) {
+	    krb5_free_keyblock (context, key);
+	    krb5_auth_con_free (context, ac);
+	    goto fail;
+	}
 
 	ret = make_pa_tgs_req(context,
 			      ac,
 			      &t->req_body, 
 			      t->padata->val,
 			      krbtgt);
-	if(ret)
+	if(ret) {
+	    krb5_free_keyblock (context, key);
+	    krb5_auth_con_free(context, ac);
 	    goto fail;
+	}
 	*subkey = key;
 	
 	krb5_auth_con_free(context, ac);
@@ -618,14 +644,15 @@ get_cred_from_kdc_flags(krb5_context context,
 	    *out_creds = calloc(1, sizeof(**out_creds));
 	    if(*out_creds == NULL)
 		ret = ENOMEM;
-	    else
+	    else {
 		ret = get_cred_kdc_la(context, ccache, flags, 
 				      in_creds, &tgts, *out_creds);
+		if (ret)
+		    free (*out_creds);
+	    }
 	    krb5_free_creds_contents(context, &tgts);
 	    krb5_free_principal(context, tmp_creds.server);
 	    krb5_free_principal(context, tmp_creds.client);
-	    if(ret)
-		free(*out_creds);
 	    return ret;
 	}
     }
@@ -670,7 +697,15 @@ get_cred_from_kdc_flags(krb5_context context,
 	
     krb5_free_principal(context, tmp_creds.server);
     krb5_free_principal(context, tmp_creds.client);
-    ret = get_cred_kdc_la(context, ccache, flags, in_creds, tgt, *out_creds);
+    *out_creds = calloc(1, sizeof(**out_creds));
+    if(*out_creds == NULL)
+	ret = ENOMEM;
+    else {
+	ret = get_cred_kdc_la(context, ccache, flags, 
+				      in_creds, tgt, *out_creds);
+	if (ret)
+	    free (*out_creds);
+    }
     krb5_free_creds(context, tgt);
     return ret;
 }
