@@ -38,7 +38,7 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: get_in_tkt.c,v 1.85 1999/04/01 18:25:20 joda Exp $");
+RCSID("$Id: get_in_tkt.c,v 1.90 1999/08/04 17:20:11 assar Exp $");
 
 krb5_error_code
 krb5_init_etype (krb5_context context,
@@ -64,7 +64,7 @@ krb5_init_etype (krb5_context context,
 	;
     *len = i;
     *val = malloc(i * sizeof(int));
-    if (*val == NULL) {
+    if (i != 0 && *val == NULL) {
 	ret = ENOMEM;
 	goto cleanup;
     }
@@ -81,7 +81,7 @@ cleanup:
 static krb5_error_code
 decrypt_tkt (krb5_context context,
 	     krb5_keyblock *key,
-	     unsigned usage,
+	     krb5_key_usage usage,
 	     krb5_const_pointer decrypt_arg,
 	     krb5_kdc_rep *dec_rep)
 {
@@ -304,12 +304,12 @@ make_pa_enc_timestamp(krb5_context context, PA_DATA *pa,
     EncryptedData encdata;
     krb5_error_code ret;
     int32_t sec, usec;
-    unsigned usec2;
+    int usec2;
     krb5_crypto crypto;
     
     krb5_us_timeofday (context, &sec, &usec);
     p.patimestamp = sec;
-    usec2 = usec;
+    usec2         = usec;
     p.pausec      = &usec2;
 
     ret = encode_PA_ENC_TS_ENC(buf + sizeof(buf) - 1,
@@ -352,33 +352,47 @@ add_padata(krb5_context context,
 	   krb5_principal client,
 	   krb5_key_proc key_proc,
 	   krb5_const_pointer keyseed,
-	   krb5_enctype enctype, 
+	   krb5_enctype *enctypes, 
+	   unsigned netypes,
 	   krb5_salt *salt)
 {
     krb5_error_code ret;
     PA_DATA *pa2;
-    krb5_keyblock *key;
     krb5_salt salt2;
+    krb5_enctype *ep;
+    int i;
     
     if(salt == NULL) {
 	/* default to standard salt */
 	ret = krb5_get_pw_salt (context, client, &salt2);
 	salt = &salt2;
     }
-    ret = (*key_proc)(context, enctype, *salt, keyseed, &key);
-    if(salt == &salt2)
-	krb5_free_salt(context, salt2);
-    if (ret)
-	return ret;
-    pa2 = realloc(md->val, (md->len + 1) * sizeof(*md->val));
-    if(pa2 == NULL)
+    if (!enctypes) {
+	enctypes = context->etypes; /* XXX */
+	netypes = 0;
+	for (ep = enctypes; *ep != ETYPE_NULL; ep++)
+	    netypes++;
+    }
+    pa2 = realloc (md->val, (md->len + netypes) * sizeof(*md->val));
+    if (pa2 == NULL)
 	return ENOMEM;
     md->val = pa2;
-    ret = make_pa_enc_timestamp(context, &md->val[md->len], enctype, key);
-    krb5_free_keyblock (context, key);
-    if(ret)
-	return ret;
-    md->len++;
+
+    for (i = 0; i < netypes; ++i) {
+	krb5_keyblock *key;
+
+	ret = (*key_proc)(context, enctypes[i], *salt, keyseed, &key);
+	if (ret)
+	    continue;
+	ret = make_pa_enc_timestamp (context, &md->val[md->len],
+				     enctypes[i], key);
+	krb5_free_keyblock (context, key);
+	if (ret)
+	    return ret;
+	++md->len;
+    }
+    if(salt == &salt2)
+	krb5_free_salt(context, salt2);
     return 0;
 }
 
@@ -397,7 +411,6 @@ init_as_req (krb5_context context,
 {
     krb5_error_code ret;
     krb5_salt salt;
-    krb5_enctype etype;
 
     memset(a, 0, sizeof(*a));
 
@@ -452,21 +465,27 @@ init_as_req (krb5_context context,
     if (ret)
 	goto fail;
 
-    etype = a->req_body.etype.val[0]; /* XXX */
+    /*
+     * This means no addresses
+     */
 
-    a->req_body.addresses = malloc(sizeof(*a->req_body.addresses));
-    if (a->req_body.addresses == NULL) {
-	ret = ENOMEM;
-	goto fail;
+    if (addrs && addrs->len == 0) {
+	a->req_body.addresses = NULL;
+    } else {
+	a->req_body.addresses = malloc(sizeof(*a->req_body.addresses));
+	if (a->req_body.addresses == NULL) {
+	    ret = ENOMEM;
+	    goto fail;
+	}
+
+	if (addrs)
+	    ret = krb5_copy_addresses(context, addrs, a->req_body.addresses);
+	else
+	    ret = krb5_get_all_client_addrs (context, a->req_body.addresses);
+	if (ret)
+	    return ret;
     }
 
-    if (addrs)
-	ret = krb5_copy_addresses(context, addrs, a->req_body.addresses);
-    else
-	ret = krb5_get_all_client_addrs (context, a->req_body.addresses);
-    if (ret)
-	return ret;
-
     a->req_body.enc_authorization_data = NULL;
     a->req_body.additional_tickets = NULL;
 
@@ -504,7 +523,7 @@ init_as_req (krb5_context context,
 			    krb5_data_zero(&salt.saltvalue);
 		    add_padata(context, a->padata, creds->client, 
 			       key_proc, keyseed, 
-			       preauth->val[i].info.val[j].etype,
+			       &preauth->val[i].info.val[j].etype, 1,
 			       sp);
 		}
 	    }
@@ -524,13 +543,15 @@ init_as_req (krb5_context context,
 
 	/* make a v5 salted pa-data */
 	add_padata(context, a->padata, creds->client, 
-		   key_proc, keyseed, etype, NULL);
+		   key_proc, keyseed, a->req_body.etype.val,
+		   a->req_body.etype.len, NULL);
 	
 	/* make a v4 salted pa-data */
 	salt.salttype = KRB5_PW_SALT;
 	krb5_data_zero(&salt.saltvalue);
 	add_padata(context, a->padata, creds->client, 
-		   key_proc, keyseed, etype, &salt);
+		   key_proc, keyseed, a->req_body.etype.val,
+		   a->req_body.etype.len, &salt);
     } else {
 	ret = KRB5_PREAUTH_BAD_TYPE;
 	goto fail;
