@@ -13,10 +13,15 @@
 #endif
 #endif
 
+#if !defined(__GNUC__) && !defined(__attribute__)
+#define __attribute__(x)
+#endif
+
 krb5_error_code
 krb524_convert_creds_kdc __P((
 	krb5_context context,
-	krb5_creds *v5creds,
+	krb5_ccache ccache,
+	krb5_creds *in_cred,
 	struct credentials *v4creds));
 
 krb5_error_code
@@ -45,6 +50,21 @@ krb5_524_conv_principal __P((
 	char *instance,
 	char *realm));
 
+krb5_error_code
+krb5_abort __P((
+	krb5_context context,
+	krb5_error_code code,
+	const char *fmt,
+	...))
+    __attribute__ ((noreturn, format (printf, 3, 4)));
+
+krb5_error_code
+krb5_abortx __P((
+	krb5_context context,
+	const char *fmt,
+	...))
+    __attribute__ ((noreturn, format (printf, 2, 3)));
+
 krb5_error_code
 krb5_add_extra_addresses __P((
 	krb5_context context,
@@ -208,10 +228,10 @@ krb5_auth_getcksumtype __P((
 	krb5_cksumtype *cksumtype));
 
 krb5_error_code
-krb5_auth_getenctype __P((
+krb5_auth_getkeytype __P((
 	krb5_context context,
 	krb5_auth_context auth_context,
-	krb5_enctype *etype));
+	krb5_keytype *keytype));
 
 krb5_error_code
 krb5_auth_getlocalseqnumber __P((
@@ -232,10 +252,10 @@ krb5_auth_setcksumtype __P((
 	krb5_cksumtype cksumtype));
 
 krb5_error_code
-krb5_auth_setenctype __P((
+krb5_auth_setkeytype __P((
 	krb5_context context,
 	krb5_auth_context auth_context,
-	krb5_enctype etype));
+	krb5_keytype keytype));
 
 krb5_error_code
 krb5_auth_setlocalseqnumber __P((
@@ -308,7 +328,7 @@ krb5_cc_close __P((
 krb5_error_code
 krb5_cc_copy_cache __P((
 	krb5_context context,
-	krb5_ccache from,
+	const krb5_ccache from,
 	krb5_ccache to));
 
 krb5_error_code
@@ -316,7 +336,7 @@ krb5_cc_default __P((
 	krb5_context context,
 	krb5_ccache *id));
 
-char*
+const char*
 krb5_cc_default_name __P((krb5_context context));
 
 krb5_error_code
@@ -327,7 +347,7 @@ krb5_cc_destroy __P((
 krb5_error_code
 krb5_cc_end_seq_get __P((
 	krb5_context context,
-	krb5_ccache id,
+	const krb5_ccache id,
 	krb5_cc_cursor *cursor));
 
 krb5_error_code
@@ -336,7 +356,7 @@ krb5_cc_gen_new __P((
 	const krb5_cc_ops *ops,
 	krb5_ccache *id));
 
-char*
+const char*
 krb5_cc_get_name __P((
 	krb5_context context,
 	krb5_ccache id));
@@ -347,6 +367,16 @@ krb5_cc_get_principal __P((
 	krb5_ccache id,
 	krb5_principal *principal));
 
+const char*
+krb5_cc_get_type __P((
+	krb5_context context,
+	krb5_ccache id));
+
+krb5_error_code
+krb5_cc_get_version __P((
+	krb5_context context,
+	const krb5_ccache id));
+
 krb5_error_code
 krb5_cc_initialize __P((
 	krb5_context context,
@@ -356,7 +386,7 @@ krb5_cc_initialize __P((
 krb5_error_code
 krb5_cc_next_cred __P((
 	krb5_context context,
-	krb5_ccache id,
+	const krb5_ccache id,
 	krb5_creds *creds,
 	krb5_cc_cursor *cursor));
 
@@ -376,7 +406,7 @@ krb5_cc_remove_cred __P((
 krb5_error_code
 krb5_cc_resolve __P((
 	krb5_context context,
-	const char *residual,
+	const char *name,
 	krb5_ccache *id));
 
 krb5_error_code
@@ -384,7 +414,7 @@ krb5_cc_retrieve_cred __P((
 	krb5_context context,
 	krb5_ccache id,
 	krb5_flags whichfields,
-	krb5_creds *mcreds,
+	const krb5_creds *mcreds,
 	krb5_creds *creds));
 
 krb5_error_code
@@ -396,7 +426,7 @@ krb5_cc_set_flags __P((
 krb5_error_code
 krb5_cc_start_seq_get __P((
 	krb5_context context,
-	krb5_ccache id,
+	const krb5_ccache id,
 	krb5_cc_cursor *cursor));
 
 krb5_error_code
@@ -439,11 +469,13 @@ krb5_boolean
 krb5_compare_creds __P((
 	krb5_context context,
 	krb5_flags whichfields,
-	krb5_creds *mcreds,
-	krb5_creds *creds));
+	const krb5_creds *mcreds,
+	const krb5_creds *creds));
 
 krb5_error_code
-krb5_config_file_free __P((krb5_config_section *s));
+krb5_config_file_free __P((
+	krb5_context context,
+	krb5_config_section *s));
 
 void
 krb5_config_free_strings __P((char **strings));
@@ -525,6 +557,13 @@ krb5_config_parse_file __P((
 	const char *fname,
 	krb5_config_section **res));
 
+krb5_error_code
+krb5_config_parse_file_debug __P((
+	const char *fname,
+	krb5_config_section **res,
+	unsigned *lineno,
+	char **error_message));
+
 const void *
 krb5_config_vget __P((
 	krb5_context context,
@@ -627,6 +666,12 @@ krb5_copy_data __P((
 	const krb5_data *indata,
 	krb5_data **outdata));
 
+krb5_error_code
+krb5_copy_host_realm __P((
+	krb5_context context,
+	const krb5_realm *from,
+	krb5_realm **to));
+
 krb5_error_code
 krb5_copy_keyblock __P((
 	krb5_context context,
@@ -778,7 +823,8 @@ krb5_decrypt_ticket __P((
 	krb5_context context,
 	Ticket *ticket,
 	krb5_keyblock *key,
-	EncTicketPart *out));
+	EncTicketPart *out,
+	krb5_flags flags));
 
 krb5_error_code
 krb5_domain_x500_decode __P((
@@ -887,7 +933,7 @@ krb5_enctype_valid __P((
 	krb5_enctype etype));
 
 krb5_boolean
-krb5_enctypes_comptible_keys __P((
+krb5_enctypes_compatible_keys __P((
 	krb5_context context,
 	krb5_enctype etype1,
 	krb5_enctype etype2));
@@ -898,14 +944,22 @@ krb5_err __P((
 	int eval,
 	krb5_error_code code,
 	const char *fmt,
-	...));
+	...))
+    __attribute__ ((noreturn, format (printf, 4, 5)));
 
 krb5_error_code
 krb5_errx __P((
 	krb5_context context,
 	int eval,
 	const char *fmt,
-	...));
+	...))
+    __attribute__ ((noreturn, format (printf, 3, 4)));
+
+krb5_error_code
+krb5_expand_hostname __P((
+	krb5_context context,
+	const char *orig_hostname,
+	char **new_hostname));
 
 PA_DATA *
 krb5_find_padata __P((
@@ -1007,6 +1061,17 @@ krb5_free_ticket __P((
 	krb5_context context,
 	krb5_ticket *ticket));
 
+krb5_error_code
+krb5_fwd_tgt_creds __P((
+	krb5_context context,
+	krb5_auth_context auth_context,
+	const char *hostname,
+	krb5_principal client,
+	krb5_principal server,
+	krb5_ccache ccache,
+	int forwardable,
+	krb5_data *out_data));
+
 void
 krb5_generate_random_block __P((
 	void *buf,
@@ -1080,16 +1145,26 @@ krb5_get_default_realm __P((
 	krb5_context context,
 	krb5_realm *realm));
 
+krb5_error_code
+krb5_get_default_realms __P((
+	krb5_context context,
+	krb5_realm **realms));
+
 const char *
 krb5_get_err_text __P((
 	krb5_context context,
-	long code));
+	krb5_error_code code));
 
 krb5_error_code
 krb5_get_extra_addresses __P((
 	krb5_context context,
 	krb5_addresses *addresses));
 
+krb5_error_code
+krb5_get_fcache_version __P((
+	krb5_context context,
+	int *version));
+
 krb5_error_code
 krb5_get_forwarded_creds __P((
 	krb5_context context,
@@ -1106,6 +1181,12 @@ krb5_get_host_realm __P((
 	const char *host,
 	krb5_realm **realms));
 
+krb5_error_code
+krb5_get_host_realm_int __P((
+	krb5_context context,
+	const char *host,
+	krb5_realm **realms));
+
 krb5_error_code
 krb5_get_in_cred __P((
 	krb5_context context,
@@ -1270,6 +1351,12 @@ krb5_get_pw_salt __P((
 krb5_boolean
 krb5_get_use_admin_kdc __P((krb5_context context));
 
+size_t
+krb5_get_wrapped_length __P((
+	krb5_context context,
+	krb5_crypto crypto,
+	size_t data_len));
+
 int
 krb5_getportbyname __P((
 	krb5_context context,
@@ -1326,6 +1413,20 @@ krb5_keytab_key_proc __P((
 	krb5_const_pointer keyseed,
 	krb5_keyblock **key));
 
+krb5_error_code
+krb5_keytype_to_enctypes __P((
+	krb5_context context,
+	krb5_keytype keytype,
+	unsigned *len,
+	int **val));
+
+krb5_error_code
+krb5_keytype_to_enctypes_default __P((
+	krb5_context context,
+	krb5_keytype keytype,
+	unsigned *len,
+	int **val));
+
 krb5_error_code
 krb5_keytype_to_string __P((
 	krb5_context context,
@@ -1343,6 +1444,14 @@ krb5_kt_close __P((
 	krb5_context context,
 	krb5_keytab id));
 
+krb5_boolean
+krb5_kt_compare __P((
+	krb5_context context,
+	krb5_keytab_entry *entry,
+	krb5_const_principal principal,
+	krb5_kvno vno,
+	krb5_enctype enctype));
+
 krb5_error_code
 krb5_kt_copy_entry_contents __P((
 	krb5_context context,
@@ -1406,7 +1515,7 @@ krb5_kt_read_service_key __P((
 krb5_error_code
 krb5_kt_register __P((
 	krb5_context context,
-	krb5_kt_ops *ops));
+	const krb5_kt_ops *ops));
 
 krb5_error_code
 krb5_kt_remove_entry __P((
@@ -1438,7 +1547,8 @@ krb5_log __P((
 	krb5_log_facility *fac,
 	int level,
 	const char *fmt,
-	...));
+	...))
+    __attribute__((format (printf, 4, 5)));
 
 krb5_error_code
 krb5_log_msg __P((
@@ -1447,7 +1557,14 @@ krb5_log_msg __P((
 	int level,
 	char **reply,
 	const char *fmt,
-	...));
+	...))
+    __attribute__((format (printf, 5, 6)));
+
+krb5_error_code
+krb5_make_addrport __P((
+	krb5_address **res,
+	const krb5_address *addr,
+	int16_t port));
 
 krb5_error_code
 krb5_make_principal __P((
@@ -1693,6 +1810,18 @@ krb5_recvauth __P((
 	krb5_keytab keytab,
 	krb5_ticket **ticket));
 
+krb5_error_code
+krb5_recvauth_match_version __P((
+	krb5_context context,
+	krb5_auth_context *auth_context,
+	krb5_pointer p_fd,
+	krb5_boolean (*match_appl_version)(void *, const char*),
+	void *match_data,
+	krb5_principal server,
+	int32_t flags,
+	krb5_keytab keytab,
+	krb5_ticket **ticket));
+
 krb5_error_code
 krb5_ret_address __P((
 	krb5_storage *sp,
@@ -1708,6 +1837,11 @@ krb5_ret_authdata __P((
 	krb5_storage *sp,
 	krb5_authdata *auth));
 
+krb5_error_code
+krb5_ret_creds __P((
+	krb5_storage *sp,
+	krb5_creds *creds));
+
 krb5_error_code
 krb5_ret_data __P((
 	krb5_storage *sp,
@@ -1798,6 +1932,11 @@ krb5_set_extra_addresses __P((
 	krb5_context context,
 	krb5_addresses *addresses));
 
+krb5_error_code
+krb5_set_fcache_version __P((
+	krb5_context context,
+	int version));
+
 void
 krb5_set_use_admin_kdc __P((
 	krb5_context context,
@@ -1829,6 +1968,11 @@ krb5_sockaddr2address __P((
 	const struct sockaddr *sa,
 	krb5_address *addr));
 
+krb5_error_code
+krb5_sockaddr2port __P((
+	const struct sockaddr *sa,
+	int16_t *port));
+
 krb5_boolean
 krb5_sockaddr_uninteresting __P((const struct sockaddr *sa));
 
@@ -1838,6 +1982,11 @@ krb5_std_usage __P((
 	struct getargs *args,
 	int num_args));
 
+void
+krb5_storage_clear_flags __P((
+	krb5_storage *sp,
+	krb5_flags flags));
+
 krb5_storage *
 krb5_storage_emem __P((void));
 
@@ -1855,6 +2004,16 @@ krb5_storage_from_mem __P((
 	void *buf,
 	size_t len));
 
+krb5_boolean
+krb5_storage_is_flags __P((
+	krb5_storage *sp,
+	krb5_flags flags));
+
+void
+krb5_storage_set_flags __P((
+	krb5_storage *sp,
+	krb5_flags flags));
+
 krb5_error_code
 krb5_storage_to_data __P((
 	krb5_storage *sp,
@@ -1875,6 +2034,11 @@ krb5_store_authdata __P((
 	krb5_storage *sp,
 	krb5_authdata auth));
 
+krb5_error_code
+krb5_store_creds __P((
+	krb5_storage *sp,
+	krb5_creds *creds));
+
 krb5_error_code
 krb5_store_data __P((
 	krb5_storage *sp,
@@ -1972,6 +2136,13 @@ krb5_string_to_keytype __P((
 	const char *string,
 	krb5_keytype *keytype));
 
+krb5_error_code
+krb5_string_to_salttype __P((
+	krb5_context context,
+	krb5_enctype etype,
+	const char *string,
+	krb5_salttype *salttype));
+
 krb5_error_code
 krb5_timeofday __P((
 	krb5_context context,
@@ -2009,6 +2180,21 @@ krb5_us_timeofday __P((
 	int32_t *sec,
 	int32_t *usec));
 
+krb5_error_code
+krb5_vabort __P((
+	krb5_context context,
+	krb5_error_code code,
+	const char *fmt,
+	va_list ap))
+    __attribute__ ((noreturn, format (printf, 3, 0)));
+
+krb5_error_code
+krb5_vabortx __P((
+	krb5_context context,
+	const char *fmt,
+	va_list ap))
+    __attribute__ ((noreturn, format (printf, 2, 0)));
+
 krb5_error_code
 krb5_verify_ap_req __P((
 	krb5_context context,
@@ -2016,6 +2202,7 @@ krb5_verify_ap_req __P((
 	krb5_ap_req *ap_req,
 	krb5_const_principal server,
 	krb5_keyblock *keyblock,
+	krb5_flags flags,
 	krb5_flags *ap_req_options,
 	krb5_ticket **ticket));
 
@@ -2061,20 +2248,31 @@ krb5_verify_user __P((
 	krb5_boolean secure,
 	const char *service));
 
+krb5_error_code
+krb5_verify_user_lrealm __P((
+	krb5_context context,
+	krb5_principal principal,
+	krb5_ccache ccache,
+	const char *password,
+	krb5_boolean secure,
+	const char *service));
+
 krb5_error_code
 krb5_verr __P((
 	krb5_context context,
 	int eval,
 	krb5_error_code code,
 	const char *fmt,
-	va_list ap));
+	va_list ap))
+    __attribute__ ((noreturn, format (printf, 4, 0)));
 
 krb5_error_code
 krb5_verrx __P((
 	krb5_context context,
 	int eval,
 	const char *fmt,
-	va_list ap));
+	va_list ap))
+    __attribute__ ((noreturn, format (printf, 3, 0)));
 
 krb5_error_code
 krb5_vlog __P((
@@ -2082,7 +2280,8 @@ krb5_vlog __P((
 	krb5_log_facility *fac,
 	int level,
 	const char *fmt,
-	va_list ap));
+	va_list ap))
+    __attribute__((format (printf, 4, 0)));
 
 krb5_error_code
 krb5_vlog_msg __P((
@@ -2091,33 +2290,38 @@ krb5_vlog_msg __P((
 	char **reply,
 	int level,
 	const char *fmt,
-	va_list ap));
+	va_list ap))
+    __attribute__((format (printf, 5, 0)));
 
 krb5_error_code
 krb5_vwarn __P((
 	krb5_context context,
 	krb5_error_code code,
 	const char *fmt,
-	va_list ap));
+	va_list ap))
+    __attribute__ ((format (printf, 3, 0)));
 
 krb5_error_code
 krb5_vwarnx __P((
 	krb5_context context,
 	const char *fmt,
-	va_list ap));
+	va_list ap))
+    __attribute__ ((format (printf, 2, 0)));
 
 krb5_error_code
 krb5_warn __P((
 	krb5_context context,
 	krb5_error_code code,
 	const char *fmt,
-	...));
+	...))
+    __attribute__ ((format (printf, 3, 4)));
 
 krb5_error_code
 krb5_warnx __P((
 	krb5_context context,
 	const char *fmt,
-	...));
+	...))
+    __attribute__ ((format (printf, 2, 3)));
 
 krb5_error_code
 krb5_write_message __P((
