@@ -36,7 +36,7 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: krb5.h,v 1.141 1999/04/01 18:23:14 joda Exp $ */
+/* $Id: krb5.h,v 1.156 1999/11/04 01:58:06 assar Exp $ */
 
 #ifndef __KRB5_H__
 #define __KRB5_H__
@@ -85,6 +85,7 @@ typedef enum krb5_cksumtype {
   CKSUMTYPE_RSA_MD5		= 7,
   CKSUMTYPE_RSA_MD5_DES		= 8,
   CKSUMTYPE_RSA_MD5_DES3	= 9,
+/*  CKSUMTYPE_SHA1		= 10,*/
   CKSUMTYPE_HMAC_SHA1_DES3	= 12,
   CKSUMTYPE_SHA1		= 1000 /* correct value? */
 } krb5_cksumtype;
@@ -96,17 +97,11 @@ typedef enum krb5_enctype {
   ETYPE_DES_CBC_MD4		= 2,
   ETYPE_DES_CBC_MD5		= 3,
   ETYPE_DES3_CBC_MD5		= 5,
-#if NEW_DES3_CODE
-  ETYPE_NEW_DES3_CBC_SHA1	= 7,
-  ETYPE_DES3_CBC_SHA1		= ETYPE_NEW_DES3_CBC_SHA1,
-#else
   ETYPE_OLD_DES3_CBC_SHA1	= 7,
-  ETYPE_NEW_DES3_CBC_SHA1	= 99,
-  ETYPE_DES3_CBC_SHA1		= ETYPE_OLD_DES3_CBC_SHA1,
-#endif
   ETYPE_SIGN_DSA_GENERATE	= 8,
   ETYPE_ENCRYPT_RSA_PRIV	= 9,
   ETYPE_ENCRYPT_RSA_PUB		= 10,
+  ETYPE_DES3_CBC_SHA1		= 16, /* with key derivation */
   ETYPE_ENCTYPE_PK_CROSS	= 48,
   ETYPE_DES_CBC_NONE		= 0x1000,
   ETYPE_DES3_CBC_NONE		= 0x1001
@@ -200,8 +195,10 @@ typedef struct krb5_preauthdata {
 }krb5_preauthdata;
 
 typedef enum krb5_address_type { 
-    KRB5_ADDRESS_INET  = 2,
-    KRB5_ADDRESS_INET6 = 24
+    KRB5_ADDRESS_INET     =   2,
+    KRB5_ADDRESS_INET6    =  24,
+    KRB5_ADDRESS_ADDRPORT = 256,
+    KRB5_ADDRESS_IPPORT   = 257
 } krb5_address_type;
 
 enum {
@@ -227,18 +224,13 @@ struct krb5_cc_ops;
 
 #define KRB5_DEFAULT_CCROOT "FILE:/tmp/krb5cc_"
 
+typedef void *krb5_cc_cursor;
+
 typedef struct krb5_ccache_data {
-    struct krb5_cc_ops *ops;
+    const struct krb5_cc_ops *ops;
     krb5_data data;
 }krb5_ccache_data;
 
-typedef struct krb5_cc_cursor {
-    union {
-	int fd;
-	void *v;
-    } u;
-}krb5_cc_cursor;
-
 typedef struct krb5_ccache_data *krb5_ccache;
 
 typedef struct krb5_context_data *krb5_context;
@@ -284,6 +276,10 @@ typedef union {
     krb5_flags i;
 } krb5_kdc_flags;
 
+/* flags for krb5_verify_ap_req */
+
+#define KRB5_VERIFY_AP_REQ_IGNORE_INVALID	(1 << 0)
+
 #define KRB5_GC_CACHED		1
 #define KRB5_GC_USER_USER	2
 
@@ -326,6 +322,7 @@ typedef struct krb5_cc_ops {
     krb5_error_code (*remove_cred)(krb5_context, krb5_ccache, 
 				   krb5_flags, krb5_creds*);
     krb5_error_code (*set_flags)(krb5_context, krb5_ccache, krb5_flags);
+    int (*get_version)(krb5_context, krb5_ccache);
 } krb5_cc_ops;
 
 struct krb5_log_facility;
@@ -347,7 +344,8 @@ typedef krb5_config_binding krb5_config_section;
 
 typedef struct krb5_context_data {
     krb5_enctype *etypes;
-    char *default_realm;
+    krb5_enctype *etypes_des;
+    char **default_realms;
     time_t max_skew;
     time_t kdc_timeout;
     unsigned max_retries;
@@ -357,13 +355,20 @@ typedef struct krb5_context_data {
     struct et_list *et_list;
     struct krb5_log_facility *warn_dest;
     krb5_cc_ops *cc_ops;
-    int num_ops;
+    int num_cc_ops;
     const char *http_proxy;
     const char *time_fmt;
     krb5_boolean log_utc;
     const char *default_keytab;
     krb5_boolean use_admin_kdc;
     krb5_addresses *extra_addresses;
+    krb5_boolean srv_lookup;		/* do SRV lookups */
+    krb5_boolean srv_try_txt;		/* try TXT records also */
+    krb5_boolean srv_try_rfc2052;	/* try RFC2052 compatible records */
+    int32_t fcache_vno;			/* create cache files w/ this
+                                           version */
+    int num_kt_types;			/* # of registered keytab types */
+    struct krb5_keytab_data *kt_types;  /* registered keytab types */
 } krb5_context_data;
 
 enum {
@@ -390,12 +395,18 @@ struct krb5_rcache_data;
 typedef struct krb5_rcache_data *krb5_rcache;
 typedef Authenticator krb5_donot_reply;
 
+#define KRB5_STORAGE_HOST_BYTEORDER			0x01
+#define KRB5_STORAGE_PRINCIPAL_WRONG_NUM_COMPONENTS	0x02
+#define KRB5_STORAGE_PRINCIPAL_NO_NAME_TYPE		0x04
+#define KRB5_STORAGE_KEYBLOCK_KEYTYPE_TWICE		0x08
+
 typedef struct krb5_storage {
     void *data;
-    size_t (*fetch)(struct krb5_storage*, void*, size_t);
-    size_t (*store)(struct krb5_storage*, void*, size_t);
+    ssize_t (*fetch)(struct krb5_storage*, void*, size_t);
+    ssize_t (*store)(struct krb5_storage*, void*, size_t);
     off_t (*seek)(struct krb5_storage*, off_t, int);
     void (*free)(struct krb5_storage*);
+    krb5_flags flags;
 } krb5_storage;
 
 typedef struct krb5_keytab_entry {
@@ -407,7 +418,7 @@ typedef struct krb5_keytab_entry {
 typedef struct krb5_kt_cursor {
     int fd;
     krb5_storage *sp;
-    off_t offset;
+    void *data;
 } krb5_kt_cursor;
 
 struct krb5_keytab_data;
@@ -428,6 +439,7 @@ struct krb5_keytab_data {
     krb5_error_code (*add)(krb5_context, krb5_keytab, krb5_keytab_entry*);
     krb5_error_code (*remove)(krb5_context, krb5_keytab, krb5_keytab_entry*);
     void *data;
+    int32_t version;
 };
 
 typedef struct krb5_keytab_data krb5_kt_ops;
@@ -451,6 +463,8 @@ typedef struct krb5_auth_context_data {
 
     krb5_address *local_address;
     krb5_address *remote_address;
+    int16_t local_port;
+    int16_t remote_port;
     krb5_keyblock *keyblock;
     krb5_keyblock *local_subkey;
     krb5_keyblock *remote_subkey;
@@ -463,6 +477,9 @@ typedef struct krb5_auth_context_data {
     krb5_pointer i_vector;
   
     krb5_rcache rcache;
+
+    krb5_keytype keytype;	/* requested key type ? */
+    krb5_cksumtype cksumtype;	/* requested checksum type! */
   
 }krb5_auth_context_data, *krb5_auth_context;
 
@@ -559,6 +576,10 @@ typedef struct _krb5_verify_init_creds_opt {
 extern const krb5_cc_ops krb5_fcc_ops;
 extern const krb5_cc_ops krb5_mcc_ops;
 
+extern const krb5_kt_ops krb5_fkt_ops;
+extern const krb5_kt_ops krb5_mkt_ops;
+extern const krb5_kt_ops krb5_akf_ops;
+
 #define KRB5_KPASSWD_SUCCESS	0
 #define KRB5_KPASSWD_MALFORMED	0
 #define KRB5_KPASSWD_HARDERROR	0
